@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,11 +27,13 @@
 
 #if ENABLE(B3_JIT)
 
-#include "B3OpaqueByproduct.h"
+#include "JITOpaqueByproduct.h"
+#include <wtf/TZoneMalloc.h>
 
 namespace JSC { namespace B3 {
 
 class DataSection final : public OpaqueByproduct {
+    WTF_MAKE_TZONE_ALLOCATED(DataSection);
 public:
     DataSection(size_t size);
     ~DataSection() final;
