@@ -29,6 +29,8 @@
 
 #include "B3Value.h"
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC { namespace B3 {
 
 class JS_EXPORT_PRIVATE UpsilonValue final : public Value {
@@ -71,4 +73,6 @@ private:
 
 } } // namespace JSC::B3
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
+
 #endif // ENABLE(B3_JIT)
