@@ -89,11 +89,6 @@ unsigned Inst::jsHash() const
 void Inst::dump(PrintStream& out) const
 {
     out.print(kind, " ", listDump(args));
-    if (origin) {
-        if (args.size())
-            out.print(", ");
-        out.print(*origin);
-    }
 }
 
 } } } // namespace JSC::B3::Air
