@@ -27,11 +27,11 @@
 
 #include "CacheableIdentifier.h"
 #include "CallVariant.h"
-#include "PutKind.h"
 
 namespace JSC {
 
 class CallLinkInfo;
+class DirectCallLinkInfo;
 class OptimizingCallLinkInfo;
 class StructureStubInfo;
 
@@ -41,18 +41,31 @@ enum class GetByKind {
     TryById,
     ByIdWithThis,
     ByIdDirect,
+    ByValWithThis,
     PrivateName,
     PrivateNameById,
 };
 
 enum class PutByKind {
-    ById,
-    ByVal,
+    ByIdStrict,
+    ByIdSloppy,
+    ByValStrict,
+    ByValSloppy,
+    ByIdDirectStrict,
+    ByIdDirectSloppy,
+    ByValDirectStrict,
+    ByValDirectSloppy,
+    DefinePrivateNameById,
+    DefinePrivateNameByVal,
+    SetPrivateNameById,
+    SetPrivateNameByVal,
 };
 
 enum class DelByKind {
-    ById,
-    ByVal
+    ByIdStrict,
+    ByIdSloppy,
+    ByValStrict,
+    ByValSloppy,
 };
 
 enum class InByKind {
@@ -61,21 +74,20 @@ enum class InByKind {
     PrivateName
 };
 
-void repatchArrayGetByVal(JSGlobalObject*, CodeBlock*, JSValue base, JSValue index, StructureStubInfo&);
+void repatchArrayGetByVal(JSGlobalObject*, CodeBlock*, JSValue base, JSValue index, StructureStubInfo&, GetByKind);
 void repatchGetBy(JSGlobalObject*, CodeBlock*, JSValue, CacheableIdentifier, const PropertySlot&, StructureStubInfo&, GetByKind);
-void repatchArrayPutByVal(JSGlobalObject*, CodeBlock*, JSValue base, JSValue index, StructureStubInfo&, PutKind, ECMAMode);
-void repatchPutBy(JSGlobalObject*, CodeBlock*, JSValue, Structure*, CacheableIdentifier, const PutPropertySlot&, StructureStubInfo&, PutByKind, PutKind);
+void repatchArrayPutByVal(JSGlobalObject*, CodeBlock*, JSValue base, JSValue index, StructureStubInfo&, PutByKind);
+void repatchPutBy(JSGlobalObject*, CodeBlock*, JSValue, Structure*, CacheableIdentifier, const PutPropertySlot&, StructureStubInfo&, PutByKind);
 void repatchDeleteBy(JSGlobalObject*, CodeBlock*, DeletePropertySlot&, JSValue, Structure*, CacheableIdentifier, StructureStubInfo&, DelByKind, ECMAMode);
+void repatchArrayInByVal(JSGlobalObject*, CodeBlock*, JSValue base, JSValue index, StructureStubInfo&, InByKind);
 void repatchInBy(JSGlobalObject*, CodeBlock*, JSObject*, CacheableIdentifier, bool wasFound, const PropertySlot&, StructureStubInfo&, InByKind);
 void repatchHasPrivateBrand(JSGlobalObject*, CodeBlock*, JSObject*, CacheableIdentifier, bool wasFound,  StructureStubInfo&);
 void repatchCheckPrivateBrand(JSGlobalObject*, CodeBlock*, JSObject*, CacheableIdentifier, StructureStubInfo&);
 void repatchSetPrivateBrand(JSGlobalObject*, CodeBlock*, JSObject*, Structure*, CacheableIdentifier, StructureStubInfo&);
 void repatchInstanceOf(JSGlobalObject*, CodeBlock*, JSValue, JSValue prototype, StructureStubInfo&, bool wasFound);
-void linkMonomorphicCall(VM&, CallFrame*, CallLinkInfo&, CodeBlock*, JSObject* callee, MacroAssemblerCodePtr<JSEntryPtrTag>);
-void linkDirectCall(CallFrame*, OptimizingCallLinkInfo&, CodeBlock*, MacroAssemblerCodePtr<JSEntryPtrTag>);
-void linkSlowFor(CallFrame*, CallLinkInfo&);
-void unlinkCall(VM&, CallLinkInfo&);
-void linkPolymorphicCall(JSGlobalObject*, CallFrame*, CallLinkInfo&, CallVariant);
+void linkMonomorphicCall(VM&, JSCell*, CallLinkInfo&, CodeBlock*, JSObject* callee, CodePtr<JSEntryPtrTag>);
+void linkDirectCall(DirectCallLinkInfo&, CodeBlock*, CodePtr<JSEntryPtrTag>);
+void linkPolymorphicCall(VM&, JSCell*, CallFrame*, CallLinkInfo&, CallVariant);
 void resetGetBy(CodeBlock*, StructureStubInfo&, GetByKind);
 void resetPutBy(CodeBlock*, StructureStubInfo&, PutByKind);
 void resetDelBy(CodeBlock*, StructureStubInfo&, DelByKind);
@@ -84,8 +96,12 @@ void resetHasPrivateBrand(CodeBlock*, StructureStubInfo&);
 void resetInstanceOf(CodeBlock*, StructureStubInfo&);
 void resetCheckPrivateBrand(CodeBlock*, StructureStubInfo&);
 void resetSetPrivateBrand(CodeBlock*, StructureStubInfo&);
-void ftlThunkAwareRepatchCall(CodeBlock*, CodeLocationCall<JSInternalPtrTag>, FunctionPtr<CFunctionPtrTag> newCalleeFunction);
-MacroAssemblerCodePtr<JSEntryPtrTag> jsToWasmICCodePtr(CodeSpecializationKind, JSObject* callee);
 
+void repatchGetBySlowPathCall(CodeBlock*, StructureStubInfo&, GetByKind);
+void repatchPutBySlowPathCall(CodeBlock*, StructureStubInfo&, PutByKind);
+void repatchInBySlowPathCall(CodeBlock*, StructureStubInfo&, InByKind);
+
+void ftlThunkAwareRepatchCall(CodeBlock*, CodeLocationCall<JSInternalPtrTag>, CodePtr<CFunctionPtrTag> newCalleeFunction);
+CodePtr<JSEntryPtrTag> jsToWasmICCodePtr(CodeSpecializationKind, JSObject* callee);
 
 } // namespace JSC
