@@ -85,6 +85,10 @@ class RegisterID
             "pc"
         when "t5"
             "t5"
+        when "t6"
+            "t6"
+        when "t7"
+            "t7"
         when "csr0"
             "pcBase"
         when "csr1"
@@ -562,7 +566,7 @@ end
 def cloopEmitCallSlowPath(operands)
     $asm.putc "{"
     $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
-    $asm.putc "    SlowPathReturnType result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump});"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump});"
     $asm.putc "    decodeResult(result, t0, t1);"
     $asm.putc "}"
 end
@@ -575,7 +579,7 @@ end
 def cloopEmitCallSlowPath3(operands)
     $asm.putc "{"
     $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
-    $asm.putc "    SlowPathReturnType result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump});"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump});"
     $asm.putc "    decodeResult(result, t0, t1);"
     $asm.putc "}"
 end
@@ -583,7 +587,7 @@ end
 def cloopEmitCallSlowPath4(operands)
     $asm.putc "{"
     $asm.putc "    cloopStack.setCurrentStackPointer(sp.vp());"
-    $asm.putc "    SlowPathReturnType result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump}, #{operands[4].clDump});"
+    $asm.putc "    UGPRPair result = #{operands[0].cLabel}(#{operands[1].clDump}, #{operands[2].clDump}, #{operands[3].clDump}, #{operands[4].clDump});"
     $asm.putc "    decodeResult(result, t0, t1);"
     $asm.putc "}"
 end
@@ -702,7 +706,12 @@ class Instruction
             $asm.putc "#{operands[1].clLValue(:double)} = #{operands[0].dblMemRef};"
         when "stored"
             $asm.putc "#{operands[1].dblMemRef} = #{operands[0].clValue(:double)};"
-
+        when "transferi"
+            $asm.putc "#{operands[1].uint32MemRef} = #{operands[0].uint32MemRef}"
+        when "transferq"
+            $asm.putc "#{operands[1].uint64MemRef} = #{operands[0].uint64MemRef}"
+        when "transferp"
+            $asm.putc "#{operands[1].uintptrMemRef} = #{operands[0].uintptrMemRef}"
         when "addd"
             cloopEmitOperation(operands, :double, "+")
         when "divd"
@@ -1166,7 +1175,7 @@ class Instruction
             $asm.putc "lr = getOpcode(llint_cloop_did_return_from_js_#{uid});"
             $asm.putc "opcode = #{operands[0].clValue(:opcode)};"
             $asm.putc "DISPATCH_OPCODE();"
-            $asm.putsLabel("llint_cloop_did_return_from_js_#{uid}", false)
+            $asm.putsLabel("llint_cloop_did_return_from_js_#{uid}", false, false, false, false)
 
         # We can't do generic function calls with an arbitrary set of args, but
         # fortunately we don't have to here. All native function calls always
@@ -1208,10 +1217,6 @@ class Instruction
         end
     end
 
-    def lowerC_LOOP_WIN
-        lowerC_LOOP
-    end
-
     def recordMetaDataC_LOOP
         $asm.codeOrigin codeOriginString if $enableCodeOriginComments
         $asm.annotation annotation if $enableInstrAnnotations && (opcode != "cloopDo")
