@@ -29,18 +29,23 @@
 #include "CodeCache.h"
 #include "Debugger.h"
 #include "VMTrapsInlines.h"
+#include <wtf/text/MakeString.h>
+
+#if PLATFORM(COCOA)
+#include <wtf/cocoa/RuntimeApplicationChecksCocoa.h>
+#endif
 
 namespace JSC {
 
 const ClassInfo ProgramExecutable::s_info = { "ProgramExecutable"_s, &Base::s_info, nullptr, nullptr, CREATE_METHOD_TABLE(ProgramExecutable) };
 
 ProgramExecutable::ProgramExecutable(JSGlobalObject* globalObject, const SourceCode& source)
-    : Base(globalObject->vm().programExecutableStructure.get(), globalObject->vm(), source, false, DerivedContextType::None, false, false, EvalContextType::None, NoIntrinsic)
+    : Base(globalObject->vm().programExecutableStructure.get(), globalObject->vm(), source, NoLexicallyScopedFeatures, DerivedContextType::None, false, false, EvalContextType::None, NoIntrinsic)
 {
     ASSERT(source.provider()->sourceType() == SourceProviderSourceType::Program);
     VM& vm = globalObject->vm();
     if (vm.typeProfiler() || vm.controlFlowProfiler())
-        vm.functionHasExecutedCache()->insertUnexecutedRange(sourceID(), typeProfilingStartOffset(vm), typeProfilingEndOffset(vm));
+        vm.functionHasExecutedCache()->insertUnexecutedRange(sourceID(), typeProfilingStartOffset(), typeProfilingEndOffset());
 }
 
 void ProgramExecutable::destroy(JSCell* cell)
@@ -64,6 +69,17 @@ static GlobalPropertyLookUpStatus hasRestrictedGlobalProperty(JSGlobalObject* gl
     return GlobalPropertyLookUpStatus::NonConfigurable;
 }
 
+static ALWAYS_INLINE bool requiresCanDeclareGlobalFunctionQuirk()
+{
+#if PLATFORM(COCOA)
+    // https://bugs.webkit.org/show_bug.cgi?id=267199
+    static bool requiresQuirk = !linkedOnOrAfterSDKWithBehavior(SDKAlignedBehavior::ThrowIfCanDeclareGlobalFunctionFails);
+    return requiresQuirk;
+#else
+    return false;
+#endif
+}
+
 JSObject* ProgramExecutable::initializeGlobalProperties(VM& vm, JSGlobalObject* globalObject, JSScope* scope)
 {
     DeferTermination deferScope(vm);
@@ -74,10 +90,9 @@ JSObject* ProgramExecutable::initializeGlobalProperties(VM& vm, JSGlobalObject*
     ASSERT(&globalObject->vm() == &vm);
 
     ParserError error;
-    JSParserStrictMode strictMode = isInStrictContext() ? JSParserStrictMode::Strict : JSParserStrictMode::NotStrict;
     OptionSet<CodeGenerationMode> codeGenerationMode = globalObject->defaultCodeGenerationMode();
     UnlinkedProgramCodeBlock* unlinkedCodeBlock = vm.codeCache()->getUnlinkedProgramCodeBlock(
-        vm, this, source(), strictMode, codeGenerationMode, error);
+        vm, this, source(), codeGenerationMode, error);
 
     if (globalObject->hasDebugger())
         globalObject->debugger()->sourceParsed(globalObject, source().provider(), error.line(), error.message());
@@ -87,32 +102,43 @@ JSObject* ProgramExecutable::initializeGlobalProperties(VM& vm, JSGlobalObject*
 
     JSValue nextPrototype = globalObject->getPrototypeDirect();
     while (nextPrototype && nextPrototype.isObject()) {
-        if (UNLIKELY(asObject(nextPrototype)->type() == ProxyObjectType))
+        if (asObject(nextPrototype)->type() == ProxyObjectType) [[unlikely]]
             return createTypeError(globalObject, "Proxy is not allowed in the global prototype chain."_s);
         nextPrototype = asObject(nextPrototype)->getPrototypeDirect();
     }
     
     JSGlobalLexicalEnvironment* globalLexicalEnvironment = globalObject->globalLexicalEnvironment();
+    bool hasGlobalLexicalDeclarations = !globalLexicalEnvironment->isEmpty();
     const VariableEnvironment& variableDeclarations = unlinkedCodeBlock->variableDeclarations();
     const VariableEnvironment& lexicalDeclarations = unlinkedCodeBlock->lexicalDeclarations();
+    size_t numberOfFunctions = unlinkedCodeBlock->numberOfFunctionDecls();
     // The ES6 spec says that no vars/global properties/let/const can be duplicated in the global scope.
     // This carried out section 15.1.8 of the ES6 spec: http://www.ecma-international.org/ecma-262/6.0/index.html#sec-globaldeclarationinstantiation
     {
         // Check for intersection of "var" and "let"/"const"/"class"
-        for (auto& entry : lexicalDeclarations) {
-            if (variableDeclarations.contains(entry.key))
-                return createSyntaxError(globalObject, makeString("Can't create duplicate variable: '", String(entry.key.get()), "'"));
-        }
-
         // Check if any new "let"/"const"/"class" will shadow any pre-existing global property names (with configurable = false), or "var"/"let"/"const" variables.
         // It's an error to introduce a shadow.
         for (auto& entry : lexicalDeclarations) {
+            if (hasGlobalLexicalDeclarations) {
+                bool hasProperty = globalLexicalEnvironment->hasProperty(globalObject, entry.key.get());
+                RETURN_IF_EXCEPTION(throwScope, nullptr);
+                if (hasProperty) {
+                    if (entry.value.isConst() && !vm.globalConstRedeclarationShouldThrow() && !isInStrictContext()) [[unlikely]] {
+                        // We only allow "const" duplicate declarations under this setting.
+                        // For example, we don't allow "let" variables to be overridden by "const" variables.
+                        if (globalLexicalEnvironment->isConstVariable(entry.key.get()))
+                            continue;
+                    }
+                    return createErrorForDuplicateGlobalVariableDeclaration(globalObject, entry.key.get());
+                }
+            }
+
             // The ES6 spec says that RestrictedGlobalProperty can't be shadowed.
             GlobalPropertyLookUpStatus status = hasRestrictedGlobalProperty(globalObject, entry.key.get());
             RETURN_IF_EXCEPTION(throwScope, nullptr);
             switch (status) {
             case GlobalPropertyLookUpStatus::NonConfigurable:
-                return createSyntaxError(globalObject, makeString("Can't create duplicate variable that shadows a global property: '", String(entry.key.get()), "'"));
+                return createSyntaxError(globalObject, makeString("Can't create duplicate variable that shadows a global property: '"_s, StringView(entry.key.get()), '\''));
             case GlobalPropertyLookUpStatus::Configurable:
                 // Lexical bindings can shadow global properties if the given property's attribute is configurable.
                 // https://tc39.github.io/ecma262/#sec-globaldeclarationinstantiation step 5-c, `hasRestrictedGlobal` becomes false
@@ -124,28 +150,47 @@ JSObject* ProgramExecutable::initializeGlobalProperties(VM& vm, JSGlobalObject*
             case GlobalPropertyLookUpStatus::NotFound:
                 break;
             }
-
-            bool hasProperty = globalLexicalEnvironment->hasProperty(globalObject, entry.key.get());
-            RETURN_IF_EXCEPTION(throwScope, nullptr);
-            if (hasProperty) {
-                if (UNLIKELY(entry.value.isConst() && !vm.globalConstRedeclarationShouldThrow() && !isInStrictContext())) {
-                    // We only allow "const" duplicate declarations under this setting.
-                    // For example, we don't "let" variables to be overridden by "const" variables.
-                    if (globalLexicalEnvironment->isConstVariable(entry.key.get()))
-                        continue;
-                }
-                return createSyntaxError(globalObject, makeString("Can't create duplicate variable: '", String(entry.key.get()), "'"));
-            }
         }
 
         // Check if any new "var"s will shadow any previous "let"/"const"/"class" names.
         // It's an error to introduce a shadow.
-        if (!globalLexicalEnvironment->isEmpty()) {
+        if (hasGlobalLexicalDeclarations) {
             for (auto& entry : variableDeclarations) {
+                if (entry.value.isSloppyModeHoistedFunction())
+                    continue;
                 bool hasProperty = globalLexicalEnvironment->hasProperty(globalObject, entry.key.get());
                 RETURN_IF_EXCEPTION(throwScope, nullptr);
                 if (hasProperty)
-                    return createSyntaxError(globalObject, makeString("Can't create duplicate variable: '", String(entry.key.get()), "'"));
+                    return createErrorForDuplicateGlobalVariableDeclaration(globalObject, entry.key.get());
+            }
+        }
+
+        for (size_t i = 0; i < numberOfFunctions; ++i) {
+            UnlinkedFunctionExecutable* unlinkedFunctionExecutable = unlinkedCodeBlock->functionDecl(i);
+            ASSERT(!unlinkedFunctionExecutable->name().isEmpty());
+            bool canDeclare = globalObject->canDeclareGlobalFunction(unlinkedFunctionExecutable->name());
+            RETURN_IF_EXCEPTION(throwScope, nullptr);
+            if (!canDeclare) {
+                if (requiresCanDeclareGlobalFunctionQuirk()) {
+                    VM::DeletePropertyModeScope scope(vm, VM::DeletePropertyMode::IgnoreConfigurable);
+                    JSCell::deleteProperty(globalObject, globalObject, unlinkedFunctionExecutable->name());
+                    RETURN_IF_EXCEPTION(throwScope, nullptr);
+                    continue;
+                }
+                return createErrorForInvalidGlobalFunctionDeclaration(globalObject, unlinkedFunctionExecutable->name());
+            }
+        }
+
+        if (!globalObject->isStructureExtensible()) {
+            for (auto& entry : variableDeclarations) {
+                if (entry.value.isFunction() || entry.value.isSloppyModeHoistedFunction())
+                    continue;
+                ASSERT(entry.value.isVar());
+                const Identifier& ident = Identifier::fromUid(vm, entry.key.get());
+                bool canDeclare = globalObject->canDeclareGlobalVar(ident);
+                RETURN_IF_EXCEPTION(throwScope, nullptr);
+                if (!canDeclare)
+                    return createErrorForInvalidGlobalVarDeclaration(globalObject, ident);
             }
         }
     }
@@ -154,29 +199,56 @@ JSObject* ProgramExecutable::initializeGlobalProperties(VM& vm, JSGlobalObject*
 
     BatchedTransitionOptimizer optimizer(vm, globalObject);
 
-    for (size_t i = 0, numberOfFunctions = unlinkedCodeBlock->numberOfFunctionDecls(); i < numberOfFunctions; ++i) {
+    // https://tc39.es/ecma262/#sec-web-compat-globaldeclarationinstantiation (excluding last step)
+    if (!isInStrictContext()) {
+        for (auto& entry : variableDeclarations) {
+            if (!entry.value.isSloppyModeHoistedFunction())
+                continue;
+
+            const Identifier& ident = Identifier::fromUid(vm, entry.key.get());
+
+            if (hasGlobalLexicalDeclarations) {
+                bool hasProperty = globalLexicalEnvironment->hasProperty(globalObject, ident);
+                RETURN_IF_EXCEPTION(throwScope, nullptr);
+                if (hasProperty)
+                    continue;
+            }
+
+            bool canDeclare = globalObject->canDeclareGlobalVar(ident);
+            RETURN_IF_EXCEPTION(throwScope, nullptr);
+            if (!canDeclare)
+                continue;
+
+            globalObject->createGlobalVarBinding<BindingCreationContext::Global>(ident);
+            RETURN_IF_EXCEPTION(throwScope, nullptr);
+        }
+    }
+
+    for (size_t i = 0; i < numberOfFunctions; ++i) {
         UnlinkedFunctionExecutable* unlinkedFunctionExecutable = unlinkedCodeBlock->functionDecl(i);
         ASSERT(!unlinkedFunctionExecutable->name().isEmpty());
-        globalObject->addFunction(globalObject, unlinkedFunctionExecutable->name());
+        globalObject->createGlobalFunctionBinding<BindingCreationContext::Global>(unlinkedFunctionExecutable->name());
+        RETURN_IF_EXCEPTION(throwScope, nullptr);
         if (vm.typeProfiler() || vm.controlFlowProfiler()) {
             vm.functionHasExecutedCache()->insertUnexecutedRange(sourceID(), 
-                unlinkedFunctionExecutable->typeProfilingStartOffset(), 
-                unlinkedFunctionExecutable->typeProfilingEndOffset());
+                unlinkedFunctionExecutable->unlinkedFunctionStart(),
+                unlinkedFunctionExecutable->unlinkedFunctionEnd());
         }
     }
 
     for (auto& entry : variableDeclarations) {
+        if (entry.value.isFunction() || entry.value.isSloppyModeHoistedFunction())
+            continue;
         ASSERT(entry.value.isVar());
-        globalObject->addVar(globalObject, Identifier::fromUid(vm, entry.key.get()));
-        throwScope.assertNoException();
+        globalObject->createGlobalVarBinding<BindingCreationContext::Global>(Identifier::fromUid(vm, entry.key.get()));
+        RETURN_IF_EXCEPTION(throwScope, nullptr);
     }
 
     {
-        JSGlobalLexicalEnvironment* globalLexicalEnvironment = jsCast<JSGlobalLexicalEnvironment*>(globalObject->globalScope());
         SymbolTable* symbolTable = globalLexicalEnvironment->symbolTable();
         ConcurrentJSLocker locker(symbolTable->m_lock);
         for (auto& entry : lexicalDeclarations) {
-            if (UNLIKELY(entry.value.isConst() && !vm.globalConstRedeclarationShouldThrow() && !isInStrictContext())) {
+            if (entry.value.isConst() && !vm.globalConstRedeclarationShouldThrow() && !isInStrictContext()) [[unlikely]] {
                 if (symbolTable->contains(locker, entry.key.get()))
                     continue;
             }
