@@ -26,26 +26,10 @@
 #include "config.h"
 #include "WasmCompilationMode.h"
 
+#include "Options.h"
 #include <wtf/Assertions.h>
 
 namespace JSC { namespace Wasm {
 
-const char* makeString(CompilationMode mode)
-{
-    switch (mode) {
-    case CompilationMode::LLIntMode:
-        return "LLInt";
-    case CompilationMode::BBQMode:
-        return "BBQ";
-    case CompilationMode::OMGMode:
-        return "OMG";
-    case CompilationMode::OMGForOSREntryMode:
-        return "OMGForOSREntry";
-    case CompilationMode::EmbedderEntrypointMode:
-        return "EmbedderEntrypoint";
-    }
-    RELEASE_ASSERT_NOT_REACHED();
-    return "";
-}
 
 } } // namespace JSC::Wasm
