@@ -33,7 +33,7 @@ enum TFailCode
 
 static void usage();
 static sh::GLenum FindShaderType(const char *fileName);
-static bool CompileFile(char *fileName, ShHandle compiler, ShCompileOptions compileOptions);
+static bool CompileFile(char *fileName, ShHandle compiler, const ShCompileOptions &compileOptions);
 static void LogMsg(const char *msg, const char *name, const int num, const char *logName);
 static void PrintVariable(const std::string &prefix, size_t index, const sh::ShaderVariable &var);
 static void PrintActiveVariables(ShHandle compiler);
@@ -79,23 +79,24 @@ int main(int argc, char *argv[])
 {
     TFailCode failCode = ESuccess;
 
-    ShCompileOptions compileOptions = 0;
+    ShCompileOptions compileOptions = {};
     int numCompiles                 = 0;
     ShHandle vertexCompiler         = 0;
     ShHandle fragmentCompiler       = 0;
     ShHandle computeCompiler        = 0;
     ShHandle geometryCompiler       = 0;
+    ShHandle tessEvalCompiler       = 0;
+    ShHandle tessControlCompiler    = 0;
     ShShaderSpec spec               = SH_GLES2_SPEC;
     ShShaderOutput output           = SH_ESSL_OUTPUT;
 
-#if defined(ANGLE_ENABLE_VULKAN)
-    sh::InitializeGlslang();
-#endif
     sh::Initialize();
 
     ShBuiltInResources resources;
     GenerateResources(&resources);
 
+    bool printActiveVariables = false;
+
     argc--;
     argv++;
     for (; (argc >= 1) && (failCode == ESuccess); argc--, argv++)
@@ -105,13 +106,13 @@ int main(int argc, char *argv[])
             switch (argv[0][1])
             {
                 case 'i':
-                    compileOptions |= SH_INTERMEDIATE_TREE;
+                    compileOptions.intermediateTree = true;
                     break;
                 case 'o':
-                    compileOptions |= SH_OBJECT_CODE;
+                    compileOptions.objectCode = true;
                     break;
                 case 'u':
-                    compileOptions |= SH_VARIABLES;
+                    printActiveVariables = true;
                     break;
                 case 's':
                     if (argv[0][2] == '=')
@@ -125,6 +126,10 @@ int main(int argc, char *argv[])
                                     {
                                         spec = SH_GLES3_1_SPEC;
                                     }
+                                    else if (argv[0][5] == '2')
+                                    {
+                                        spec = SH_GLES3_2_SPEC;
+                                    }
                                     else
                                     {
                                         spec = SH_GLES3_SPEC;
@@ -154,16 +159,6 @@ int main(int argc, char *argv[])
                                     resources.FragmentPrecisionHigh = 1;
                                 }
                                 break;
-                            case 'd':
-                                if (argv[0][4] == 'c')
-                                {
-                                    spec = SH_GL_COMPATIBILITY_SPEC;
-                                }
-                                else
-                                {
-                                    spec = SH_GL_CORE_SPEC;
-                                }
-                                break;
                             default:
                                 failCode = EFailUsage;
                         }
@@ -179,19 +174,19 @@ int main(int argc, char *argv[])
                         switch (argv[0][3])
                         {
                             case 'e':
-                                output = SH_ESSL_OUTPUT;
-                                compileOptions |= SH_INITIALIZE_UNINITIALIZED_LOCALS;
+                                output                                       = SH_ESSL_OUTPUT;
+                                compileOptions.initializeUninitializedLocals = true;
                                 break;
                             case 'g':
                                 if (!ParseGLSLOutputVersion(&argv[0][sizeof("-b=g") - 1], &output))
                                 {
                                     failCode = EFailUsage;
                                 }
-                                compileOptions |= SH_INITIALIZE_UNINITIALIZED_LOCALS;
+                                compileOptions.initializeUninitializedLocals = true;
                                 break;
                             case 'v':
                                 output = SH_SPIRV_VULKAN_OUTPUT;
-                                compileOptions |= SH_INITIALIZE_UNINITIALIZED_LOCALS;
+                                compileOptions.initializeUninitializedLocals = true;
                                 break;
                             case 'h':
                                 if (argv[0][4] == '1' && argv[0][5] == '1')
@@ -204,14 +199,7 @@ int main(int argc, char *argv[])
                                 }
                                 break;
                             case 'm':
-                                if (argv[0][4] == 'v')
-                                {
-                                    output = SH_SPIRV_METAL_OUTPUT;
-                                }
-                                else
-                                {
-                                    output = SH_MSL_METAL_OUTPUT;
-                                }
+                                output = SH_MSL_METAL_OUTPUT;
                                 break;
                             default:
                                 failCode = EFailUsage;
@@ -261,8 +249,8 @@ int main(int argc, char *argv[])
                       case 'm':
                           resources.OVR_multiview2 = 1;
                           resources.OVR_multiview = 1;
-                          compileOptions |= SH_INITIALIZE_BUILTINS_FOR_INSTANCED_MULTIVIEW;
-                          compileOptions |= SH_SELECT_VIEW_IN_NV_GLSL_VERTEX_SHADER;
+                          compileOptions.initializeBuiltinsForInstancedMultiview = true;
+                          compileOptions.selectViewInNvGLSLVertexShader = true;
                           break;
                       case 'y': resources.EXT_YUV_target = 1; break;
                       case 's': resources.OES_sample_variables = 1; break;
@@ -317,11 +305,32 @@ int main(int argc, char *argv[])
                 case GL_GEOMETRY_SHADER_EXT:
                     if (geometryCompiler == 0)
                     {
+                        resources.EXT_geometry_shader = 1;
                         geometryCompiler =
                             sh::ConstructCompiler(GL_GEOMETRY_SHADER_EXT, spec, output, &resources);
                     }
                     compiler = geometryCompiler;
                     break;
+                case GL_TESS_CONTROL_SHADER_EXT:
+                    if (tessControlCompiler == 0)
+                    {
+                        assert(spec == SH_GLES3_1_SPEC || spec == SH_GLES3_2_SPEC);
+                        resources.EXT_tessellation_shader = 1;
+                        tessControlCompiler = sh::ConstructCompiler(GL_TESS_CONTROL_SHADER_EXT,
+                                                                    spec, output, &resources);
+                    }
+                    compiler = tessControlCompiler;
+                    break;
+                case GL_TESS_EVALUATION_SHADER_EXT:
+                    if (tessEvalCompiler == 0)
+                    {
+                        assert(spec == SH_GLES3_1_SPEC || spec == SH_GLES3_2_SPEC);
+                        resources.EXT_tessellation_shader = 1;
+                        tessEvalCompiler = sh::ConstructCompiler(GL_TESS_EVALUATION_SHADER_EXT,
+                                                                 spec, output, &resources);
+                    }
+                    compiler = tessEvalCompiler;
+                    break;
                 default:
                     break;
             }
@@ -331,8 +340,7 @@ int main(int argc, char *argv[])
                 {
                     case SH_HLSL_3_0_OUTPUT:
                     case SH_HLSL_4_1_OUTPUT:
-                    case SH_HLSL_4_0_FL9_3_OUTPUT:
-                        compileOptions &= ~SH_SELECT_VIEW_IN_NV_GLSL_VERTEX_SHADER;
+                        compileOptions.selectViewInNvGLSLVertexShader = false;
                         break;
                     default:
                         break;
@@ -346,7 +354,7 @@ int main(int argc, char *argv[])
                 LogMsg("END", "COMPILER", numCompiles, "INFO LOG");
                 printf("\n\n");
 
-                if (compiled && (compileOptions & SH_OBJECT_CODE))
+                if (compiled && compileOptions.objectCode)
                 {
                     LogMsg("BEGIN", "COMPILER", numCompiles, "OBJ CODE");
                     if (output != SH_SPIRV_VULKAN_OUTPUT)
@@ -362,7 +370,7 @@ int main(int argc, char *argv[])
                     LogMsg("END", "COMPILER", numCompiles, "OBJ CODE");
                     printf("\n\n");
                 }
-                if (compiled && (compileOptions & SH_VARIABLES))
+                if (compiled && printActiveVariables)
                 {
                     LogMsg("BEGIN", "COMPILER", numCompiles, "VARIABLES");
                     PrintActiveVariables(compiler);
@@ -381,24 +389,41 @@ int main(int argc, char *argv[])
     }
 
     if ((vertexCompiler == 0) && (fragmentCompiler == 0) && (computeCompiler == 0) &&
-        (geometryCompiler == 0))
+        (geometryCompiler == 0) && (tessControlCompiler == 0) && (tessEvalCompiler == 0))
+    {
         failCode = EFailUsage;
+    }
     if (failCode == EFailUsage)
+    {
         usage();
+    }
 
     if (vertexCompiler)
+    {
         sh::Destruct(vertexCompiler);
+    }
     if (fragmentCompiler)
+    {
         sh::Destruct(fragmentCompiler);
+    }
     if (computeCompiler)
+    {
         sh::Destruct(computeCompiler);
+    }
     if (geometryCompiler)
+    {
         sh::Destruct(geometryCompiler);
+    }
+    if (tessControlCompiler)
+    {
+        sh::Destruct(tessControlCompiler);
+    }
+    if (tessEvalCompiler)
+    {
+        sh::Destruct(tessEvalCompiler);
+    }
 
     sh::Finalize();
-#if defined(ANGLE_ENABLE_VULKAN)
-    sh::FinalizeGlslang();
-#endif
 
     return failCode;
 }
@@ -411,18 +436,17 @@ void usage()
     // clang-format off
     printf(
         "Usage: translate [-i -o -u -l -b=e -b=g -b=h9 -x=i -x=d] file1 file2 ...\n"
-        "Where: filename : filename ending in .frag or .vert\n"
+        "Where: filename : filename ending in .frag*, .vert*, .comp*, .geom*, .tcs* or .tes*\n"
         "       -i       : print intermediate tree\n"
         "       -o       : print translated code\n"
         "       -u       : print active attribs, uniforms, varyings and program outputs\n"
         "       -s=e2    : use GLES2 spec (this is by default)\n"
         "       -s=e3    : use GLES3 spec\n"
         "       -s=e31   : use GLES31 spec (in development)\n"
+        "       -s=e32   : use GLES32 spec (in development)\n"
         "       -s=w     : use WebGL 1.0 spec\n"
         "       -s=wn    : use WebGL 1.0 spec with no highp support in fragment shaders\n"
         "       -s=w2    : use WebGL 2.0 spec\n"
-        "       -s=d     : use Desktop Core spec (in development)\n"
-        "       -s=dc    : use Desktop Compatibility spec (in development)\n"
         "       -b=e     : output GLSL ES code (this is by default)\n"
         "       -b=g     : output GLSL code (compatibility profile)\n"
         "       -b=g[NUM]: output GLSL code (NUM can be 130, 140, 150, 330, 400, 410, 420, 430, "
@@ -431,7 +455,6 @@ void usage()
         "       -b=h9    : output HLSL9 code\n"
         "       -b=h11   : output HLSL11 code\n"
         "       -b=m     : output MSL code (direct)\n"
-        "       -b=mv    : output MSL code (via SPIR-V)\n"
         "       -x=i     : enable GL_OES_EGL_image_external\n"
         "       -x=d     : enable GL_OES_EGL_standard_derivatives\n"
         "       -x=r     : enable ARB_texture_rectangle\n"
@@ -454,6 +477,10 @@ void usage()
 //
 //   .frag*    = fragment shader
 //   .vert*    = vertex shader
+//   .comp*    = compute shader
+//   .geom*    = geometry shader
+//   .tcs*     = tessellation control shader
+//   .tes*     = tessellation evaluation shader
 //
 sh::GLenum FindShaderType(const char *fileName)
 {
@@ -476,6 +503,10 @@ sh::GLenum FindShaderType(const char *fileName)
             return GL_COMPUTE_SHADER;
         if (strncmp(ext, ".geom", 5) == 0)
             return GL_GEOMETRY_SHADER_EXT;
+        if (strncmp(ext, ".tcs", 5) == 0)
+            return GL_TESS_CONTROL_SHADER_EXT;
+        if (strncmp(ext, ".tes", 5) == 0)
+            return GL_TESS_EVALUATION_SHADER_EXT;
     }
 
     return GL_FRAGMENT_SHADER;
@@ -484,7 +515,7 @@ sh::GLenum FindShaderType(const char *fileName)
 //
 //   Read a file's data into a string, and compile it using sh::Compile
 //
-bool CompileFile(char *fileName, ShHandle compiler, ShCompileOptions compileOptions)
+bool CompileFile(char *fileName, ShHandle compiler, const ShCompileOptions &compileOptions)
 {
     ShaderSource source;
     if (!ReadShaderSource(fileName, source))
@@ -884,7 +915,9 @@ static void PrintSpirv(const sh::BinaryBlob &blob)
     spvtools::SpirvTools spirvTools(SPV_ENV_VULKAN_1_1);
 
     std::string readableSpirv;
-    spirvTools.Disassemble(blob, &readableSpirv, 0);
+    spirvTools.Disassemble(blob, &readableSpirv,
+                           SPV_BINARY_TO_TEXT_OPTION_COMMENT | SPV_BINARY_TO_TEXT_OPTION_INDENT |
+                               SPV_BINARY_TO_TEXT_OPTION_NESTED_INDENT);
 
     puts(readableSpirv.c_str());
 #endif
