@@ -15,6 +15,7 @@
 #include "libANGLE/Error.h"
 #include "libANGLE/ImageIndex.h"
 #include "libANGLE/Observer.h"
+#include "libANGLE/angletypes.h"
 #include "libANGLE/formatutils.h"
 #include "libANGLE/renderer/FramebufferAttachmentObjectImpl.h"
 
@@ -43,12 +44,6 @@ class FramebufferAttachmentObject;
 class Renderbuffer;
 class Texture;
 
-enum class InitState
-{
-    MayNeedInit,
-    Initialized,
-};
-
 // FramebufferAttachment implements a GL framebuffer attachment.
 // Attachments are "light" containers, which store pointers to ref-counted GL objects.
 // We support GL texture (2D/3D/Cube/2D array) and renderbuffer object attachments.
@@ -64,14 +59,15 @@ class FramebufferAttachment final
                           GLenum type,
                           GLenum binding,
                           const ImageIndex &textureIndex,
-                          FramebufferAttachmentObject *resource);
+                          FramebufferAttachmentObject *resource,
+                          rx::UniqueSerial framebufferSerial);
 
     FramebufferAttachment(FramebufferAttachment &&other);
     FramebufferAttachment &operator=(FramebufferAttachment &&other);
 
     ~FramebufferAttachment();
 
-    void detach(const Context *context);
+    void detach(const Context *context, rx::UniqueSerial framebufferSerial);
     void attach(const Context *context,
                 GLenum type,
                 GLenum binding,
@@ -80,7 +76,8 @@ class FramebufferAttachment final
                 GLsizei numViews,
                 GLuint baseViewIndex,
                 bool isMultiview,
-                GLsizei samples);
+                GLsizei samples,
+                rx::UniqueSerial framebufferSerial);
 
     // Helper methods
     GLuint getRedSize() const;
@@ -92,12 +89,14 @@ class FramebufferAttachment final
     GLenum getComponentType() const;
     GLenum getColorEncoding() const;
 
-    bool isBoundAsSamplerOrImage(ContextID contextID) const;
-
     bool isTextureWithId(TextureID textureId) const
     {
         return mType == GL_TEXTURE && id() == textureId.value;
     }
+    bool isExternalTexture() const
+    {
+        return mType == GL_TEXTURE && getTextureImageIndex().getType() == gl::TextureType::External;
+    }
     bool isRenderbufferWithId(GLuint renderbufferId) const
     {
         return mType == GL_RENDERBUFFER && id() == renderbufferId;
@@ -118,7 +117,8 @@ class FramebufferAttachment final
     bool isMultiview() const;
     GLint getBaseViewIndex() const;
 
-    GLsizei getRenderToTextureSamples() const { return mRenderToTextureSamples; }
+    bool isRenderToTexture() const;
+    GLsizei getRenderToTextureSamples() const;
 
     // The size of the underlying resource the attachment points to. The 'depth' value will
     // correspond to a 3D texture depth or the layer count of a 2D array texture. For Surfaces and
@@ -132,13 +132,26 @@ class FramebufferAttachment final
     GLenum type() const { return mType; }
     bool isAttached() const { return mType != GL_NONE; }
     bool isRenderable(const Context *context) const;
+    bool isYUV() const;
+    // Checks whether the attachment is an external image such as AHB or dmabuf, where
+    // synchronization is done without individually naming the objects that are involved.  This
+    // excludes Vulkan images (EXT_external_objects) as they are individually listed during
+    // synchronization.
+    //
+    // This function is used to disable optimizations that involve deferring operations, as there is
+    // no efficient way to perform those deferred operations on sync if the specific objects are
+    // unknown.
+    bool isExternalImageWithoutIndividualSync() const;
+    bool hasFrontBufferUsage() const;
+    bool hasFoveatedRendering() const;
+    const gl::FoveationState *getFoveationState() const;
 
     Renderbuffer *getRenderbuffer() const;
     Texture *getTexture() const;
     const egl::Surface *getSurface() const;
     FramebufferAttachmentObject *getResource() const;
     InitState initState() const;
-    angle::Result initializeContents(const Context *context);
+    angle::Result initializeContents(const Context *context) const;
     void setInitState(InitState initState) const;
 
     // "T" must be static_castable from FramebufferAttachmentRenderTarget
@@ -159,6 +172,8 @@ class FramebufferAttachment final
     static const GLint kDefaultRenderToTextureSamples;
 
   private:
+    bool isSpecified() const;
+
     angle::Result getRenderTargetImpl(const Context *context,
                                       GLsizei samples,
                                       rx::FramebufferAttachmentRenderTarget **rtOut) const;
@@ -191,6 +206,14 @@ class FramebufferAttachment final
     GLsizei mNumViews;
     bool mIsMultiview;
     GLint mBaseViewIndex;
+    // A single-sampled texture can be attached to a framebuffer either as single-sampled or as
+    // multisampled-render-to-texture.  In the latter case, |mRenderToTextureSamples| will contain
+    // the number of samples.  For renderbuffers, the number of samples is inherited from the
+    // renderbuffer itself.
+    //
+    // Note that textures cannot change storage between single and multisample once attached to a
+    // framebuffer.  Renderbuffers instead can, and caching the number of renderbuffer samples here
+    // can lead to stale data.
     GLsizei mRenderToTextureSamples;
 };
 
@@ -201,20 +224,29 @@ class FramebufferAttachmentObject : public angle::Subject, public angle::Observe
     FramebufferAttachmentObject();
     ~FramebufferAttachmentObject() override;
 
+    virtual bool isAttachmentSpecified(const ImageIndex &imageIndex) const                 = 0;
     virtual Extents getAttachmentSize(const ImageIndex &imageIndex) const                  = 0;
     virtual Format getAttachmentFormat(GLenum binding, const ImageIndex &imageIndex) const = 0;
     virtual GLsizei getAttachmentSamples(const ImageIndex &imageIndex) const               = 0;
     virtual bool isRenderable(const Context *context,
                               GLenum binding,
                               const ImageIndex &imageIndex) const                          = 0;
+    virtual bool isYUV() const                                                             = 0;
+    virtual bool isExternalImageWithoutIndividualSync() const                              = 0;
+    virtual bool hasFrontBufferUsage() const                                               = 0;
+    virtual bool hasProtectedContent() const                                               = 0;
+    virtual bool hasFoveatedRendering() const                                              = 0;
+    virtual const gl::FoveationState *getFoveationState() const                            = 0;
 
-    virtual void onAttach(const Context *context) = 0;
-    virtual void onDetach(const Context *context) = 0;
-    virtual GLuint getId() const                  = 0;
+    virtual void onAttach(const Context *context, rx::UniqueSerial framebufferSerial) = 0;
+    virtual void onDetach(const Context *context, rx::UniqueSerial framebufferSerial) = 0;
+    virtual GLuint getId() const                                                      = 0;
 
     // These are used for robust resource initialization.
-    virtual InitState initState(const ImageIndex &imageIndex) const              = 0;
-    virtual void setInitState(const ImageIndex &imageIndex, InitState initState) = 0;
+    virtual InitState initState(GLenum binding, const ImageIndex &imageIndex) const = 0;
+    virtual void setInitState(GLenum binding,
+                              const ImageIndex &imageIndex,
+                              InitState initState)                                  = 0;
 
     angle::Result getAttachmentRenderTarget(const Context *context,
                                             GLenum binding,
@@ -222,7 +254,9 @@ class FramebufferAttachmentObject : public angle::Subject, public angle::Observe
                                             GLsizei samples,
                                             rx::FramebufferAttachmentRenderTarget **rtOut) const;
 
-    angle::Result initializeContents(const Context *context, const ImageIndex &imageIndex);
+    angle::Result initializeContents(const Context *context,
+                                     GLenum binding,
+                                     const ImageIndex &imageIndex);
 
   protected:
     virtual rx::FramebufferAttachmentObjectImpl *getAttachmentImpl() const = 0;
@@ -234,6 +268,12 @@ inline const ImageIndex &FramebufferAttachment::getTextureImageIndex() const
     return mTarget.textureIndex();
 }
 
+inline bool FramebufferAttachment::isSpecified() const
+{
+    ASSERT(mResource);
+    return mResource->isAttachmentSpecified(mTarget.textureIndex());
+}
+
 inline Extents FramebufferAttachment::getSize() const
 {
     ASSERT(mResource);
@@ -248,8 +288,7 @@ inline Format FramebufferAttachment::getFormat() const
 
 inline GLsizei FramebufferAttachment::getSamples() const
 {
-    return (mRenderToTextureSamples != kDefaultRenderToTextureSamples) ? getRenderToTextureSamples()
-                                                                       : getResourceSamples();
+    return isRenderToTexture() ? getRenderToTextureSamples() : getResourceSamples();
 }
 
 inline GLsizei FramebufferAttachment::getResourceSamples() const
@@ -274,6 +313,35 @@ inline bool FramebufferAttachment::isRenderable(const Context *context) const
     return mResource->isRenderable(context, mTarget.binding(), mTarget.textureIndex());
 }
 
+inline bool FramebufferAttachment::isYUV() const
+{
+    ASSERT(mResource);
+    return mResource->isYUV();
+}
+
+inline bool FramebufferAttachment::isExternalImageWithoutIndividualSync() const
+{
+    ASSERT(mResource);
+    return mResource->isExternalImageWithoutIndividualSync();
+}
+
+inline bool FramebufferAttachment::hasFrontBufferUsage() const
+{
+    ASSERT(mResource);
+    return mResource->hasFrontBufferUsage();
+}
+
+inline bool FramebufferAttachment::hasFoveatedRendering() const
+{
+    ASSERT(mResource);
+    return mResource->hasFoveatedRendering();
+}
+
+inline const gl::FoveationState *FramebufferAttachment::getFoveationState() const
+{
+    ASSERT(mResource);
+    return mResource->getFoveationState();
+}
 }  // namespace gl
 
 #endif  // LIBANGLE_FRAMEBUFFERATTACHMENT_H_
