@@ -47,22 +47,25 @@ using SupportedTimestamps       = angle::PackedEnumBitSet<Timestamp>;
 
 struct SurfaceState final : private angle::NonCopyable
 {
-    SurfaceState(const egl::Config *configIn, const AttributeMap &attributesIn);
+    SurfaceState(SurfaceID idIn, const egl::Config *configIn, const AttributeMap &attributesIn);
     ~SurfaceState();
 
     bool isRobustResourceInitEnabled() const;
     bool hasProtectedContent() const;
-    EGLint getPreferredSwapInterval() const;
+
+    SurfaceID id;
 
     EGLLabelKHR label;
     const egl::Config *config;
     AttributeMap attributes;
 
     bool timestampsEnabled;
+    bool autoRefreshEnabled;
     SupportedCompositorTiming supportedCompositorTimings;
     SupportedTimestamps supportedTimestamps;
     bool directComposition;
     EGLenum swapBehavior;
+    EGLint swapInterval;
 };
 
 class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
@@ -79,9 +82,8 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     Error makeCurrent(const gl::Context *context);
     Error unMakeCurrent(const gl::Context *context);
     Error prepareSwap(const gl::Context *context);
-    Error swap(const gl::Context *context);
-    Error swapWithDamage(const gl::Context *context, const EGLint *rects, EGLint n_rects);
-    Error swapWithFrameToken(const gl::Context *context, EGLFrameTokenANGLE frameToken);
+    Error swap(gl::Context *context);
+    Error swapWithDamage(gl::Context *context, const EGLint *rects, EGLint n_rects);
     Error postSubBuffer(const gl::Context *context,
                         EGLint x,
                         EGLint y,
@@ -97,7 +99,8 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
 
     EGLint isPostSubBufferSupported() const;
 
-    void setSwapInterval(EGLint interval);
+    void setRequestedSwapInterval(EGLint interval);
+    void setSwapInterval(const Display *display, EGLint interval);
     Error onDestroy(const Display *display);
 
     void setMipmapLevel(EGLint level);
@@ -112,17 +115,12 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     // width and height can change with client window resizing
     EGLint getWidth() const;
     EGLint getHeight() const;
-    // Note: windows cannot be resized on Android.  The approach requires
-    // calling vkGetPhysicalDeviceSurfaceCapabilitiesKHR.  However, that is
-    // expensive; and there are troublesome timing issues for other parts of
-    // ANGLE (which cause test failures and crashes).  Therefore, a
-    // special-Android-only path is created just for the querying of EGL_WIDTH
-    // and EGL_HEIGHT.
-    // https://issuetracker.google.com/issues/153329980
+    // Sizes that Surface will have after render target is first accessed (e.g. after draw).
     egl::Error getUserWidth(const egl::Display *display, EGLint *value) const;
     egl::Error getUserHeight(const egl::Display *display, EGLint *value) const;
     EGLint getPixelAspectRatio() const;
     EGLenum getRenderBuffer() const;
+    EGLenum getRequestedRenderBuffer() const;
     EGLenum getSwapBehavior() const;
     TextureFormat getTextureFormat() const;
     EGLenum getTextureTarget() const;
@@ -136,6 +134,8 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     EGLint getVerticalResolution() const;
     EGLenum getMultisampleResolve() const;
     bool hasProtectedContent() const override;
+    bool hasFoveatedRendering() const override { return false; }
+    const gl::FoveationState *getFoveationState() const override { return nullptr; }
 
     // For lock surface buffer
     EGLint getBitmapPitch() const;
@@ -147,6 +147,7 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     EGLint getLuminanceOffset() const;
     EGLint getBitmapPixelSize() const;
     EGLAttribKHR getBitmapPointer() const;
+    EGLint getCompressionRate(const egl::Display *display) const;
     egl::Error lockSurfaceKHR(const egl::Display *display, const AttributeMap &attributes);
     egl::Error unlockSurfaceKHR(const egl::Display *display);
 
@@ -158,6 +159,7 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     EGLint isFixedSize() const;
 
     // FramebufferAttachmentObject implementation
+    bool isAttachmentSpecified(const gl::ImageIndex &imageIndex) const override;
     gl::Extents getAttachmentSize(const gl::ImageIndex &imageIndex) const override;
     gl::Format getAttachmentFormat(GLenum binding, const gl::ImageIndex &imageIndex) const override;
     GLsizei getAttachmentSamples(const gl::ImageIndex &imageIndex) const override;
@@ -165,10 +167,12 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
                       GLenum binding,
                       const gl::ImageIndex &imageIndex) const override;
     bool isYUV() const override;
-    bool isCreatedWithAHB() const override;
+    bool isExternalImageWithoutIndividualSync() const override;
+    bool hasFrontBufferUsage() const override;
 
-    void onAttach(const gl::Context *context, rx::Serial framebufferSerial) override {}
-    void onDetach(const gl::Context *context, rx::Serial framebufferSerial) override {}
+    void onAttach(const gl::Context *context, rx::UniqueSerial framebufferSerial) override {}
+    void onDetach(const gl::Context *context, rx::UniqueSerial framebufferSerial) override {}
+    SurfaceID id() const { return mState.id; }
     GLuint getId() const override;
 
     EGLint getOrientation() const { return mOrientation; }
@@ -188,6 +192,9 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     void setTimestampsEnabled(bool enabled);
     bool isTimestampsEnabled() const;
 
+    // EGL_ANDROID_front_buffer_auto_refresh entry points
+    Error setAutoRefreshEnabled(bool enabled);
+
     const SupportedCompositorTiming &getSupportedCompositorTimings() const;
     Error getCompositorTiming(EGLint numTimestamps,
                               const EGLint *names,
@@ -208,6 +215,7 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     Error getBufferAge(const gl::Context *context, EGLint *age);
 
     Error setRenderBuffer(EGLint renderBuffer);
+    void setRequestedRenderBuffer(EGLint requestedRenderBuffer);
 
     bool bufferAgeQueriedSinceLastSwap() const { return mBufferAgeQueriedSinceLastSwap; }
     void setDamageRegion(const EGLint *rects, EGLint n_rects);
@@ -219,9 +227,11 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
         ASSERT(mRefCount > 0);
         mRefCount--;
     }
+    bool isReferenced() const { return mRefCount > 0; }
 
   protected:
     Surface(EGLint surfaceType,
+            SurfaceID id,
             const egl::Config *config,
             const AttributeMap &attributes,
             bool forceRobustResourceInit,
@@ -262,8 +272,11 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
     TextureFormat mTextureFormat;
     EGLenum mTextureTarget;
 
-    EGLint mPixelAspectRatio;  // Display aspect ratio
-    EGLenum mRenderBuffer;     // Render buffer
+    EGLint mPixelAspectRatio;        // Display aspect ratio
+    EGLenum mRenderBuffer;           // Render buffer
+    EGLenum mRequestedRenderBuffer;  // Requested render buffer
+
+    EGLint mRequestedSwapInterval;
 
     EGLint mOrientation;
 
@@ -288,12 +301,14 @@ class Surface : public LabeledObject, public gl::FramebufferAttachmentObject
 
     Error destroyImpl(const Display *display);
 
-    void postSwap(const gl::Context *context);
+    void postSwap(const gl::Context *context, const rx::SurfaceSwapFeedback &feedback);
     Error releaseRef(const Display *display);
 
     // ObserverInterface implementation.
     void onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMessage message) override;
 
+    Error updatePropertiesOnSwap(const gl::Context *context);
+
     gl::InitState mColorInitState;
     gl::InitState mDepthStencilInitState;
     angle::ObserverBinding mImplObserverBinding;
@@ -303,6 +318,7 @@ class WindowSurface final : public Surface
 {
   public:
     WindowSurface(rx::EGLImplFactory *implFactory,
+                  SurfaceID id,
                   const Config *config,
                   EGLNativeWindowType window,
                   const AttributeMap &attribs,
@@ -314,10 +330,12 @@ class PbufferSurface final : public Surface
 {
   public:
     PbufferSurface(rx::EGLImplFactory *implFactory,
+                   SurfaceID id,
                    const Config *config,
                    const AttributeMap &attribs,
                    bool robustResourceInit);
     PbufferSurface(rx::EGLImplFactory *implFactory,
+                   SurfaceID id,
                    const Config *config,
                    EGLenum buftype,
                    EGLClientBuffer clientBuffer,
@@ -332,6 +350,7 @@ class PixmapSurface final : public Surface
 {
   public:
     PixmapSurface(rx::EGLImplFactory *implFactory,
+                  SurfaceID id,
                   const Config *config,
                   NativePixmapType nativePixmap,
                   const AttributeMap &attribs,
