@@ -14,6 +14,7 @@
 #include "libANGLE/Stream.h"
 #include "libANGLE/angletypes.h"
 #include "libANGLE/renderer/TextureImpl.h"
+#include "libANGLE/renderer/d3d/RendererD3D.h"
 #include "libANGLE/renderer/d3d/TextureStorage.h"
 
 namespace gl
@@ -66,13 +67,18 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
                                         const gl::Extents &size,
                                         bool fixedSampleLocations) override;
 
+    angle::Result setBuffer(const gl::Context *context, GLenum internalFormat) override;
+
     angle::Result setStorageExternalMemory(const gl::Context *context,
                                            gl::TextureType type,
                                            size_t levels,
                                            GLenum internalFormat,
                                            const gl::Extents &size,
                                            gl::MemoryObject *memoryObject,
-                                           GLuint64 offset) override;
+                                           GLuint64 offset,
+                                           GLbitfield createFlags,
+                                           GLbitfield usageFlags,
+                                           const void *imageCreateInfoPNext) override;
 
     bool isImmutable() const { return mImmutable; }
 
@@ -107,13 +113,18 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
     angle::Result setBaseLevel(const gl::Context *context, GLuint baseLevel) override;
 
     angle::Result syncState(const gl::Context *context,
-                            const gl::Texture::DirtyBits &dirtyBits) override;
+                            const gl::Texture::DirtyBits &dirtyBits,
+                            gl::Command source) override;
 
     angle::Result initializeContents(const gl::Context *context,
+                                     GLenum binding,
                                      const gl::ImageIndex &imageIndex) override;
 
     GLsizei getRenderToTextureSamples();
 
+    angle::Result ensureUnorderedAccess(const gl::Context *context);
+    angle::Result onLabelUpdate(const gl::Context *context) override;
+
     // ObserverInterface implementation.
     void onSubjectStateChange(angle::SubjectIndex index, angle::SubjectMessage message) override;
 
@@ -122,6 +133,7 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
                                const gl::ImageIndex &index,
                                GLenum type,
                                const gl::PixelUnpackState &unpack,
+                               gl::Buffer *unpackBuffer,
                                const uint8_t *pixels,
                                ptrdiff_t layerOffset);
     angle::Result subImage(const gl::Context *context,
@@ -145,9 +157,12 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
                                      const gl::PixelUnpackState &unpack,
                                      const uint8_t *pixels,
                                      ptrdiff_t layerOffset);
-    bool isFastUnpackable(const gl::Buffer *unpackBuffer, GLenum sizedInternalFormat);
+    bool isFastUnpackable(const gl::Buffer *unpackBuffer,
+                          const gl::PixelUnpackState &unpack,
+                          GLenum sizedInternalFormat);
     angle::Result fastUnpackPixels(const gl::Context *context,
                                    const gl::PixelUnpackState &unpack,
+                                   gl::Buffer *unpackBuffer,
                                    const uint8_t *pixels,
                                    const gl::Box &destArea,
                                    GLenum sizedInternalFormat,
@@ -164,9 +179,11 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
     virtual bool isImageComplete(const gl::ImageIndex &index) const = 0;
 
     bool canCreateRenderTargetForImage(const gl::ImageIndex &index) const;
+    angle::Result ensureBindFlags(const gl::Context *context, BindFlags bindFlags);
     angle::Result ensureRenderTarget(const gl::Context *context);
 
-    virtual angle::Result createCompleteStorage(bool renderTarget,
+    virtual angle::Result createCompleteStorage(const gl::Context *context,
+                                                BindFlags bindFlags,
                                                 TexStoragePointer *outTexStorage) const = 0;
     virtual angle::Result setCompleteTexStorage(const gl::Context *context,
                                                 TextureStorage *newCompleteTexStorage)  = 0;
@@ -174,7 +191,8 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
                                const gl::ImageIndex &index,
                                const gl::Box &region);
 
-    angle::Result releaseTexStorage(const gl::Context *context);
+    angle::Result releaseTexStorage(const gl::Context *context,
+                                    const gl::TexLevelMask &copyStorageToImagesMask);
 
     GLuint getBaseLevel() const { return mBaseLevel; }
 
@@ -191,7 +209,7 @@ class TextureD3D : public TextureImpl, public angle::ObserverInterface
     angle::ObserverBinding mTexStorageObserverBinding;
 
   private:
-    virtual angle::Result initializeStorage(const gl::Context *context, bool renderTarget) = 0;
+    virtual angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) = 0;
 
     virtual angle::Result updateStorage(const gl::Context *context) = 0;
 
@@ -227,6 +245,7 @@ class TextureD3D_2D : public TextureD3D
                            GLenum format,
                            GLenum type,
                            const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
                            const uint8_t *pixels) override;
     angle::Result setSubImage(const gl::Context *context,
                               const gl::ImageIndex &index,
@@ -267,7 +286,7 @@ class TextureD3D_2D : public TextureD3D
                               const gl::ImageIndex &index,
                               GLenum internalFormat,
                               GLenum type,
-                              size_t sourceLevel,
+                              GLint sourceLevel,
                               bool unpackFlipY,
                               bool unpackPremultiplyAlpha,
                               bool unpackUnmultiplyAlpha,
@@ -275,7 +294,7 @@ class TextureD3D_2D : public TextureD3D
     angle::Result copySubTexture(const gl::Context *context,
                                  const gl::ImageIndex &index,
                                  const gl::Offset &destOffset,
-                                 size_t sourceLevel,
+                                 GLint sourceLevel,
                                  const gl::Box &sourceBox,
                                  bool unpackFlipY,
                                  bool unpackPremultiplyAlpha,
@@ -310,8 +329,9 @@ class TextureD3D_2D : public TextureD3D
     void markAllImagesDirty() override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outTexStorage) const override;
     angle::Result setCompleteTexStorage(const gl::Context *context,
                                         TextureStorage *newCompleteTexStorage) override;
@@ -358,6 +378,7 @@ class TextureD3D_Cube : public TextureD3D
                            GLenum format,
                            GLenum type,
                            const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
                            const uint8_t *pixels) override;
     angle::Result setSubImage(const gl::Context *context,
                               const gl::ImageIndex &index,
@@ -398,7 +419,7 @@ class TextureD3D_Cube : public TextureD3D
                               const gl::ImageIndex &index,
                               GLenum internalFormat,
                               GLenum type,
-                              size_t sourceLevel,
+                              GLint sourceLevel,
                               bool unpackFlipY,
                               bool unpackPremultiplyAlpha,
                               bool unpackUnmultiplyAlpha,
@@ -406,7 +427,7 @@ class TextureD3D_Cube : public TextureD3D
     angle::Result copySubTexture(const gl::Context *context,
                                  const gl::ImageIndex &index,
                                  const gl::Offset &destOffset,
-                                 size_t sourceLevel,
+                                 GLint sourceLevel,
                                  const gl::Box &sourceBox,
                                  bool unpackFlipY,
                                  bool unpackPremultiplyAlpha,
@@ -439,8 +460,9 @@ class TextureD3D_Cube : public TextureD3D
     void markAllImagesDirty() override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outTexStorage) const override;
     angle::Result setCompleteTexStorage(const gl::Context *context,
                                         TextureStorage *newCompleteTexStorage) override;
@@ -490,6 +512,7 @@ class TextureD3D_3D : public TextureD3D
                            GLenum format,
                            GLenum type,
                            const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
                            const uint8_t *pixels) override;
     angle::Result setSubImage(const gl::Context *context,
                               const gl::ImageIndex &index,
@@ -530,7 +553,7 @@ class TextureD3D_3D : public TextureD3D
                               const gl::ImageIndex &index,
                               GLenum internalFormat,
                               GLenum type,
-                              size_t sourceLevel,
+                              GLint sourceLevel,
                               bool unpackFlipY,
                               bool unpackPremultiplyAlpha,
                               bool unpackUnmultiplyAlpha,
@@ -538,7 +561,7 @@ class TextureD3D_3D : public TextureD3D
     angle::Result copySubTexture(const gl::Context *context,
                                  const gl::ImageIndex &index,
                                  const gl::Offset &destOffset,
-                                 size_t sourceLevel,
+                                 GLint sourceLevel,
                                  const gl::Box &sourceBox,
                                  bool unpackFlipY,
                                  bool unpackPremultiplyAlpha,
@@ -572,8 +595,9 @@ class TextureD3D_3D : public TextureD3D
     GLint getLevelZeroDepth() const override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outStorage) const override;
     angle::Result setCompleteTexStorage(const gl::Context *context,
                                         TextureStorage *newCompleteTexStorage) override;
@@ -619,6 +643,7 @@ class TextureD3D_2DArray : public TextureD3D
                            GLenum format,
                            GLenum type,
                            const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
                            const uint8_t *pixels) override;
     angle::Result setSubImage(const gl::Context *context,
                               const gl::ImageIndex &index,
@@ -659,7 +684,7 @@ class TextureD3D_2DArray : public TextureD3D
                               const gl::ImageIndex &index,
                               GLenum internalFormat,
                               GLenum type,
-                              size_t sourceLevel,
+                              GLint sourceLevel,
                               bool unpackFlipY,
                               bool unpackPremultiplyAlpha,
                               bool unpackUnmultiplyAlpha,
@@ -667,7 +692,7 @@ class TextureD3D_2DArray : public TextureD3D
     angle::Result copySubTexture(const gl::Context *context,
                                  const gl::ImageIndex &index,
                                  const gl::Offset &destOffset,
-                                 size_t sourceLevel,
+                                 GLint sourceLevel,
                                  const gl::Box &sourceBox,
                                  bool unpackFlipY,
                                  bool unpackPremultiplyAlpha,
@@ -700,8 +725,9 @@ class TextureD3D_2DArray : public TextureD3D
     void markAllImagesDirty() override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outStorage) const override;
     angle::Result setCompleteTexStorage(const gl::Context *context,
                                         TextureStorage *newCompleteTexStorage) override;
@@ -746,6 +772,7 @@ class TextureD3DImmutableBase : public TextureD3D
                            GLenum format,
                            GLenum type,
                            const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
                            const uint8_t *pixels) override;
     angle::Result setSubImage(const gl::Context *context,
                               const gl::ImageIndex &index,
@@ -816,8 +843,9 @@ class TextureD3D_External : public TextureD3DImmutableBase
     void markAllImagesDirty() override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outTexStorage) const override;
     angle::Result setCompleteTexStorage(const gl::Context *context,
                                         TextureStorage *newCompleteTexStorage) override;
@@ -865,8 +893,9 @@ class TextureD3D_2DMultisample : public TextureD3DImmutableBase
     angle::Result updateStorage(const gl::Context *context) override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outTexStorage) const override;
     angle::Result initMipmapImages(const gl::Context *context) override;
 
@@ -910,8 +939,9 @@ class TextureD3D_2DMultisampleArray : public TextureD3DImmutableBase
     angle::Result updateStorage(const gl::Context *context) override;
 
   private:
-    angle::Result initializeStorage(const gl::Context *context, bool renderTarget) override;
-    angle::Result createCompleteStorage(bool renderTarget,
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
                                         TexStoragePointer *outTexStorage) const override;
     angle::Result initMipmapImages(const gl::Context *context) override;
 
@@ -919,6 +949,101 @@ class TextureD3D_2DMultisampleArray : public TextureD3DImmutableBase
 
     GLsizei mLayerCount;
 };
+
+class TextureD3D_Buffer : public TextureD3D
+{
+  public:
+    TextureD3D_Buffer(const gl::TextureState &data, RendererD3D *renderer);
+    ~TextureD3D_Buffer() override;
+
+    ImageD3D *getImage(const gl::ImageIndex &index) const override;
+
+    angle::Result setBuffer(const gl::Context *context, GLenum internalFormat) override;
+
+    angle::Result setImage(const gl::Context *context,
+                           const gl::ImageIndex &index,
+                           GLenum internalFormat,
+                           const gl::Extents &size,
+                           GLenum format,
+                           GLenum type,
+                           const gl::PixelUnpackState &unpack,
+                           gl::Buffer *unpackBuffer,
+                           const uint8_t *pixels) override;
+    angle::Result setSubImage(const gl::Context *context,
+                              const gl::ImageIndex &index,
+                              const gl::Box &area,
+                              GLenum format,
+                              GLenum type,
+                              const gl::PixelUnpackState &unpack,
+                              gl::Buffer *unpackBuffer,
+                              const uint8_t *pixels) override;
+
+    angle::Result setCompressedImage(const gl::Context *context,
+                                     const gl::ImageIndex &index,
+                                     GLenum internalFormat,
+                                     const gl::Extents &size,
+                                     const gl::PixelUnpackState &unpack,
+                                     size_t imageSize,
+                                     const uint8_t *pixels) override;
+    angle::Result setCompressedSubImage(const gl::Context *context,
+                                        const gl::ImageIndex &index,
+                                        const gl::Box &area,
+                                        GLenum format,
+                                        const gl::PixelUnpackState &unpack,
+                                        size_t imageSize,
+                                        const uint8_t *pixels) override;
+
+    angle::Result copyImage(const gl::Context *context,
+                            const gl::ImageIndex &index,
+                            const gl::Rectangle &sourceArea,
+                            GLenum internalFormat,
+                            gl::Framebuffer *source) override;
+    angle::Result copySubImage(const gl::Context *context,
+                               const gl::ImageIndex &index,
+                               const gl::Offset &destOffset,
+                               const gl::Rectangle &sourceArea,
+                               gl::Framebuffer *source) override;
+
+    angle::Result bindTexImage(const gl::Context *context, egl::Surface *surface) override;
+    angle::Result releaseTexImage(const gl::Context *context) override;
+
+    angle::Result setEGLImageTarget(const gl::Context *context,
+                                    gl::TextureType type,
+                                    egl::Image *image) override;
+
+    angle::Result getRenderTarget(const gl::Context *context,
+                                  const gl::ImageIndex &index,
+                                  GLsizei samples,
+                                  RenderTargetD3D **outRT) override;
+
+    GLsizei getLayerCount(int level) const override;
+
+    angle::Result syncState(const gl::Context *context,
+                            const gl::Texture::DirtyBits &dirtyBits,
+                            gl::Command source) override;
+
+    gl::ImageIndexIterator imageIterator() const override;
+    gl::ImageIndex getImageIndex(GLint mip, GLint layer) const override;
+    bool isValidIndex(const gl::ImageIndex &index) const override;
+
+  protected:
+    void markAllImagesDirty() override;
+
+  private:
+    angle::Result initializeStorage(const gl::Context *context, BindFlags bindFlags) override;
+    angle::Result createCompleteStorage(const gl::Context *context,
+                                        BindFlags bindFlags,
+                                        TexStoragePointer *outTexStorage) const override;
+    angle::Result setCompleteTexStorage(const gl::Context *context,
+                                        TextureStorage *newCompleteTexStorage) override;
+
+    angle::Result updateStorage(const gl::Context *context) override;
+    angle::Result initMipmapImages(const gl::Context *context) override;
+
+    bool isImageComplete(const gl::ImageIndex &index) const override;
+
+    GLenum mInternalFormat;
+};
 }  // namespace rx
 
 #endif  // LIBANGLE_RENDERER_D3D_TEXTURED3D_H_
