@@ -42,9 +42,11 @@ class TextureStorage9 : public TextureStorage
 
     int getTopLevel() const override;
     bool isRenderTarget() const override;
+    bool isUnorderedAccess() const override { return false; }
     bool isManaged() const override;
     bool supportsNativeMipmapFunction() const override;
     int getLevelCount() const override;
+    bool isMultiplanar(const gl::Context *context) override;
 
     angle::Result setData(const gl::Context *context,
                           const gl::ImageIndex &index,
