@@ -26,23 +26,15 @@ class SamplerVk : public SamplerImpl
     void onDestroy(const gl::Context *context) override;
     angle::Result syncState(const gl::Context *context, const bool dirty) override;
 
-    const vk::Sampler &getSampler() const
+    const vk::SamplerHelper &getSampler() const
     {
-        ASSERT(mSampler.valid());
-        return mSampler.get();
-    }
-
-    Serial getSerial() const { return mSerial; }
-
-    void onSamplerAccess(vk::ResourceUseList *resourceUseList)
-    {
-        mSampler.onResourceAccess(resourceUseList);
+        ASSERT(mSampler);
+        ASSERT(mSampler->valid());
+        return *mSampler.get();
     }
 
   private:
-    vk::SamplerHelper mSampler;
-    // The serial is used for cache indexing.
-    Serial mSerial;
+    vk::SharedSamplerPtr mSampler;
 };
 
 }  // namespace rx
