@@ -21,21 +21,18 @@ namespace rx
 class WindowSurfaceVkWayland : public WindowSurfaceVk
 {
   public:
+    // Requests of new sizes from client go through this callback, but actual resize will happen
+    // before the next operation which would provoke a backbuffer to be pulled.
     static void ResizeCallback(wl_egl_window *window, void *payload);
 
     WindowSurfaceVkWayland(const egl::SurfaceState &surfaceState,
                            EGLNativeWindowType window,
                            wl_display *display);
 
-    // On Wayland, currentExtent is undefined (0xFFFFFFFF, 0xFFFFFFFF).
-    // Whatever the application sets a swapchain's imageExtent to will be the size of the window,
-    // after the first image is presented
-    egl::Error getUserWidth(const egl::Display *display, EGLint *value) const override;
-    egl::Error getUserHeight(const egl::Display *display, EGLint *value) const override;
-
   private:
-    angle::Result createSurfaceVk(vk::Context *context, gl::Extents *extentsOut) override;
-    angle::Result getCurrentWindowSize(vk::Context *context, gl::Extents *extentsOut) override;
+    angle::Result createSurfaceVk(vk::ErrorContext *context) override;
+    angle::Result getCurrentWindowSize(vk::ErrorContext *context,
+                                       gl::Extents *extentsOut) const override;
 
     wl_display *mWaylandDisplay;
     gl::Extents mExtents;
