@@ -11,14 +11,12 @@
 
 #include "common/angleutils.h"
 #include "common/debug.h"
-#include "common/platform.h"
+#include "common/platform_helpers.h"
 #include "common/string_utils.h"
 #include "gpu_info_util/SystemInfo.h"
 
 #if defined(ANGLE_PLATFORM_APPLE)
 #    include "GPUTestConfig_mac.h"
-#elif defined(ANGLE_PLATFORM_WINDOWS)
-#    include <versionhelpers.h>
 #endif
 
 namespace angle
@@ -53,86 +51,6 @@ inline bool OperatingSystemVersionNumbers(int32_t *majorVersion, int32_t *minorV
 }
 #endif
 
-// Check if the OS is any version of Windows
-inline bool IsWin()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return true;
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is Windows XP
-inline bool IsWinXP()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return IsWindowsXPOrGreater() && !IsWindowsVistaOrGreater();
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is Windows Vista
-inline bool IsWinVista()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return IsWindowsVistaOrGreater() && !IsWindows7OrGreater();
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is Windows 7
-inline bool IsWin7()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return IsWindows7OrGreater() && !IsWindows8OrGreater();
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is Windows 8
-inline bool IsWin8()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return IsWindows8OrGreater() && !IsWindows10OrGreater();
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is Windows 10
-inline bool IsWin10()
-{
-#if defined(ANGLE_PLATFORM_WINDOWS)
-    return IsWindows10OrGreater();
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is any version of iOS
-inline bool IsIOS()
-{
-#if defined(ANGLE_PLATFORM_IOS)
-    return true;
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is any version of OSX
-inline bool IsMac()
-{
-#if defined(ANGLE_PLATFORM_MACOS)
-    return true;
-#else
-    return false;
-#endif
-}
-
 // Check if the OS is a specific major and minor version of OSX
 inline bool IsMacVersion(const int32_t majorVersion, const int32_t minorVersion)
 {
@@ -250,26 +168,6 @@ inline bool IsMacMojave()
     return false;
 }
 
-// Check if the OS is any version of Linux
-inline bool IsLinux()
-{
-#if defined(ANGLE_PLATFORM_LINUX)
-    return true;
-#else
-    return false;
-#endif
-}
-
-// Check if the OS is any version of Android
-inline bool IsAndroid()
-{
-#if defined(ANGLE_PLATFORM_ANDROID)
-    return true;
-#else
-    return false;
-#endif
-}
-
 // Generic function call to populate the SystemInfo struct. This function will
 // also cache the SystemInfo struct for future calls. Returns false if the
 // struct was not fully populated. Guaranteed to set sysInfo to a valid pointer
@@ -290,7 +188,7 @@ inline bool GetGPUTestSystemInfo(SystemInfo **sysInfo)
             // high-performance GPU for tests.
             // We can call the generic GPU info collector which selects the
             // non-Intel GPU as the active one on dual-GPU machines.
-            // See https://anglebug.com/3701.
+            // See https://anglebug.com/40096612.
             if (IsMac())
             {
                 GetDualGPUInfo(sSystemInfo);
@@ -374,9 +272,15 @@ inline bool IsVMWare()
 }
 
 // Check whether the active GPU is Apple.
-inline bool IsApple()
+inline bool IsAppleGPU()
 {
-    return angle::IsApple(GetActiveGPUVendorID());
+    return angle::IsAppleGPU(GetActiveGPUVendorID());
+}
+
+// Check whether the active GPU is Qualcomm.
+inline bool IsQualcomm()
+{
+    return angle::IsQualcomm(GetActiveGPUVendorID());
 }
 
 // Check whether this is a debug build.
@@ -439,6 +343,71 @@ inline bool IsPixel4XL()
     return IsAndroidDevice("Pixel 4 XL");
 }
 
+inline bool IsPixel6()
+{
+    return IsAndroidDevice("Pixel 6");
+}
+
+inline bool IsPixel7()
+{
+    return IsAndroidDevice("Pixel 7");
+}
+
+inline bool IsOppoFlipN2()
+{
+    return IsAndroidDevice("CPH2437");
+}
+
+inline bool IsMaliG710()
+{
+    return IsPixel7() || IsOppoFlipN2();
+}
+
+inline bool IsGalaxyA23()
+{
+    return IsAndroidDevice("SM-A236U1");
+}
+
+inline bool IsGalaxyA34()
+{
+    return IsAndroidDevice("SM-A346M");
+}
+
+inline bool IsGalaxyA54()
+{
+    return IsAndroidDevice("SM-A546E");
+}
+
+inline bool IsGalaxyS22()
+{
+    return IsAndroidDevice("SM-S901B");
+}
+
+inline bool IsGalaxyS23()
+{
+    return IsAndroidDevice("SM-S911U1");
+}
+
+inline bool IsGalaxyS24Exynos()
+{
+    return IsAndroidDevice("SM-S926B");
+}
+
+inline bool IsGalaxyS24Qualcomm()
+{
+    return IsAndroidDevice("SM-S928B");
+}
+
+inline bool IsFindX6()
+{
+    return IsAndroidDevice("PGFM10");
+}
+
+inline bool IsPineapple()
+{
+    return IsAndroidDevice("Pineapple for arm64");
+}
+
 // Check whether the active GPU is a specific device based on the string device ID.
 inline bool IsDeviceIdGPU(const std::string &gpuDeviceId)
 {
@@ -493,6 +462,12 @@ inline bool IsVulkan(const GPUTestConfig::API &api)
     return (api == GPUTestConfig::kAPIVulkan) || (api == GPUTestConfig::kAPISwiftShader);
 }
 
+// Check whether the backend API has been set to ANGLE Native in the constructor
+inline bool IsNative(const GPUTestConfig::API &api)
+{
+    return (api == GPUTestConfig::kAPINative);
+}
+
 inline bool IsSwiftShader(const GPUTestConfig::API &api)
 {
     return (api == GPUTestConfig::kAPISwiftShader);
@@ -504,6 +479,11 @@ inline bool IsMetal(const GPUTestConfig::API &api)
     return (api == GPUTestConfig::kAPIMetal);
 }
 
+inline bool IsWgpu(const GPUTestConfig::API &api)
+{
+    return (api == GPUTestConfig::kAPIWgpu);
+}
+
 }  // anonymous namespace
 
 // Load all conditions in the constructor since this data will not change during a test set.
@@ -512,12 +492,12 @@ GPUTestConfig::GPUTestConfig() : GPUTestConfig(false) {}
 GPUTestConfig::GPUTestConfig(bool isSwiftShader)
 {
     mConditions[kConditionNone]            = false;
-    mConditions[kConditionWinXP]           = IsWinXP();
-    mConditions[kConditionWinVista]        = IsWinVista();
-    mConditions[kConditionWin7]            = IsWin7();
-    mConditions[kConditionWin8]            = IsWin8();
-    mConditions[kConditionWin10]           = IsWin10();
-    mConditions[kConditionWin]             = IsWin();
+    mConditions[kConditionWinXP]           = IsWindowsXP();
+    mConditions[kConditionWinVista]        = IsWindowsVista();
+    mConditions[kConditionWin7]            = IsWindows7();
+    mConditions[kConditionWin8]            = IsWindows8();
+    mConditions[kConditionWin10]           = IsWindows10OrLater();
+    mConditions[kConditionWin]             = IsWindows();
     mConditions[kConditionMacLeopard]      = IsMacLeopard();
     mConditions[kConditionMacSnowLeopard]  = IsMacSnowLeopard();
     mConditions[kConditionMacLion]         = IsMacLion();
@@ -537,7 +517,8 @@ GPUTestConfig::GPUTestConfig(bool isSwiftShader)
     mConditions[kConditionAMD]         = !isSwiftShader && IsAMD();
     mConditions[kConditionIntel]       = !isSwiftShader && IsIntel();
     mConditions[kConditionVMWare]      = !isSwiftShader && IsVMWare();
-    mConditions[kConditionApple]       = !isSwiftShader && IsApple();
+    mConditions[kConditionApple]       = !isSwiftShader && IsAppleGPU();
+    mConditions[kConditionQualcomm]    = !isSwiftShader && IsQualcomm();
     mConditions[kConditionSwiftShader] = isSwiftShader;
 
     mConditions[kConditionRelease] = IsRelease();
@@ -549,11 +530,26 @@ GPUTestConfig::GPUTestConfig(bool isSwiftShader)
     mConditions[kConditionGLES]      = true;
     mConditions[kConditionVulkan]    = true;
     mConditions[kConditionMetal]     = true;
+    mConditions[kConditionWgpu]      = true;
+    mConditions[kConditionNative]    = true;
 
     // Devices are irrelevant if we are running on SW
     mConditions[kConditionNexus5X]          = !isSwiftShader && IsNexus5X();
     mConditions[kConditionPixel2OrXL]       = !isSwiftShader && (IsPixel2() || IsPixel2XL());
     mConditions[kConditionPixel4OrXL]       = !isSwiftShader && (IsPixel4() || IsPixel4XL());
+    mConditions[kConditionPixel6]           = !isSwiftShader && (IsPixel6());
+    mConditions[kConditionPixel7]           = !isSwiftShader && (IsPixel7());
+    mConditions[kConditionFlipN2]           = !isSwiftShader && (IsOppoFlipN2());
+    mConditions[kConditionMaliG710]         = !isSwiftShader && (IsMaliG710());
+    mConditions[kConditionGalaxyA23]        = !isSwiftShader && (IsGalaxyA23());
+    mConditions[kConditionGalaxyA34]        = !isSwiftShader && (IsGalaxyA34());
+    mConditions[kConditionGalaxyA54]        = !isSwiftShader && (IsGalaxyA54());
+    mConditions[kConditionGalaxyS22]        = !isSwiftShader && (IsGalaxyS22());
+    mConditions[kConditionGalaxyS23]        = !isSwiftShader && (IsGalaxyS23());
+    mConditions[kConditionGalaxyS24Exynos]   = !isSwiftShader && (IsGalaxyS24Exynos());
+    mConditions[kConditionGalaxyS24Qualcomm] = !isSwiftShader && (IsGalaxyS24Qualcomm());
+    mConditions[kConditionFindX6]           = !isSwiftShader && (IsFindX6());
+    mConditions[kConditionPineapple]        = !isSwiftShader && IsPineapple();
     mConditions[kConditionNVIDIAQuadroP400] = !isSwiftShader && IsNVIDIAQuadroP400();
     mConditions[kConditionNVIDIAGTX1660]    = !isSwiftShader && IsNVIDIAGTX1660();
 
@@ -578,6 +574,8 @@ GPUTestConfig::GPUTestConfig(const API &api, uint32_t preRotation)
     mConditions[kConditionGLES]      = IsGLES(api);
     mConditions[kConditionVulkan]    = IsVulkan(api);
     mConditions[kConditionMetal]     = IsMetal(api);
+    mConditions[kConditionWgpu]      = IsWgpu(api);
+    mConditions[kConditionNative]    = IsNative(api);
 
     switch (preRotation)
     {
