@@ -1262,6 +1262,10 @@ void MediaSource::detachFromElement()
 {
     ALWAYS_LOG(LOGIDENTIFIER);
 
+    if (!m_isAttached) {
+        ASSERT(isClosed());
+        return;
+    }
     // 2.4.2 Detaching from a media element
     // https://rawgit.com/w3c/media-source/45627646344eea0170dd1cbc5a3d508ca751abb8/media-source-respec.html#mediasource-detach
 
