@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, Google Inc. All rights reserved.
+ * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -95,10 +95,10 @@ void BiquadProcessor::checkForDirtyCoefficients()
     }
 }
 
-void BiquadProcessor::process(const AudioBus* source, AudioBus* destination, size_t framesToProcess)
+void BiquadProcessor::process(const AudioBus& source, AudioBus& destination, size_t framesToProcess)
 {
     if (!isInitialized()) {
-        destination->zero();
+        destination.zero();
         return;
     }
         
@@ -106,7 +106,7 @@ void BiquadProcessor::process(const AudioBus* source, AudioBus* destination, siz
             
     // For each channel of our input, process using the corresponding BiquadDSPKernel into the output channel.
     for (unsigned i = 0; i < m_kernels.size(); ++i)
-        m_kernels[i]->process(source->channel(i)->span().first(framesToProcess), destination->channel(i)->mutableSpan());
+        m_kernels[i]->process(source.channel(i)->span().first(framesToProcess), destination.channel(i)->mutableSpan());
 }
 
 void BiquadProcessor::processOnlyAudioParams(size_t framesToProcess)
