@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, Google Inc. All rights reserved.
+ * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -37,7 +37,7 @@ class AudioContext;
 // De-zippering (smoothing) is applied when the gain value is changed dynamically.
 
 class GainNode final : public AudioNode {
-    WTF_MAKE_ISO_ALLOCATED(GainNode);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(GainNode);
 public:
     static ExceptionOr<Ref<GainNode>> create(BaseAudioContext& context, const GainOptions& = { });
 
@@ -59,7 +59,7 @@ private:
     explicit GainNode(BaseAudioContext&);
 
     AudioFloatArray m_sampleAccurateGainValues;
-    Ref<AudioParam> m_gain;
+    const Ref<AudioParam> m_gain;
 };
 
 } // namespace WebCore
