@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010, Google Inc. All rights reserved.
+ * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,15 +34,17 @@
 #include "AudioNodeOutput.h"
 #include "AudioUtilities.h"
 #include "ChannelCountMode.h"
+#include "ExceptionOr.h"
 #include "HRTFDatabaseLoader.h"
 #include "HRTFPanner.h"
 #include "ScriptExecutionContext.h"
-#include <wtf/IsoMallocInlines.h>
+#include <numbers>
 #include <wtf/MathExtras.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(PannerNode);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(PannerNode);
 
 static void fixNANs(double &x)
 {
@@ -107,29 +109,25 @@ PannerNode::~PannerNode()
 
 void PannerNode::process(size_t framesToProcess)
 {
-    AudioBus* destination = output(0)->bus();
+    AudioBus& destination = output(0)->bus();
 
     if (!isInitialized() || !input(0)->isConnected()) {
-        destination->zero();
+        destination.zero();
         return;
     }
 
-    AudioBus* source = input(0)->bus();
-    if (!source) {
-        destination->zero();
-        return;
-    }
+    AudioBus& source = input(0)->bus();
 
     // The audio thread can't block on this lock, so we use tryLock() instead.
     if (!m_processLock.tryLock()) {
         // Too bad - tryLock() failed. We must be in the middle of changing the panner.
-        destination->zero();
+        destination.zero();
         return;
     }
     Locker locker { AdoptLock, m_processLock };
 
     if (!m_panner) {
-        destination->zero();
+        destination.zero();
         return;
     }
 
@@ -138,7 +136,7 @@ void PannerNode::process(size_t framesToProcess)
         if (context().isOfflineContext())
             m_hrtfDatabaseLoader->waitForLoaderThreadCompletion();
         else {
-            destination->zero();
+            destination.zero();
             return;
         }
     }
@@ -158,7 +156,7 @@ void PannerNode::process(size_t framesToProcess)
     double totalGain = distanceConeGain();
 
     // Apply gain in-place.
-    destination->copyWithGainFrom(*destination, totalGain);
+    destination.copyWithGainFrom(destination, totalGain);
 }
 
 void PannerNode::processOnlyAudioParams(size_t framesToProcess)
@@ -168,56 +166,57 @@ void PannerNode::processOnlyAudioParams(size_t framesToProcess)
         return;
 
     Locker locker { AdoptLock, m_processLock };
-    float values[AudioUtilities::renderQuantumSize];
+    std::array<float, AudioUtilities::renderQuantumSize> values;
     ASSERT(framesToProcess <= AudioUtilities::renderQuantumSize);
 
-    m_positionX->calculateSampleAccurateValues(values, framesToProcess);
-    m_positionY->calculateSampleAccurateValues(values, framesToProcess);
-    m_positionZ->calculateSampleAccurateValues(values, framesToProcess);
+    auto valuesSpan = std::span { values }.first(framesToProcess);
+    m_positionX->calculateSampleAccurateValues(valuesSpan);
+    m_positionY->calculateSampleAccurateValues(valuesSpan);
+    m_positionZ->calculateSampleAccurateValues(valuesSpan);
 
-    m_orientationX->calculateSampleAccurateValues(values, framesToProcess);
-    m_orientationY->calculateSampleAccurateValues(values, framesToProcess);
-    m_orientationZ->calculateSampleAccurateValues(values, framesToProcess);
+    m_orientationX->calculateSampleAccurateValues(valuesSpan);
+    m_orientationY->calculateSampleAccurateValues(valuesSpan);
+    m_orientationZ->calculateSampleAccurateValues(valuesSpan);
 
     listener().updateValuesIfNeeded(framesToProcess);
 }
 
-void PannerNode::processSampleAccurateValues(AudioBus* destination, const AudioBus* source, size_t framesToProcess)
+void PannerNode::processSampleAccurateValues(AudioBus& destination, const AudioBus& source, size_t framesToProcess)
 {
     // Get the sample accurate values from all of the AudioParams, including the
     // values from the AudioListener.
-    float pannerX[AudioUtilities::renderQuantumSize];
-    float pannerY[AudioUtilities::renderQuantumSize];
-    float pannerZ[AudioUtilities::renderQuantumSize];
+    std::array<float, AudioUtilities::renderQuantumSize> pannerX;
+    std::array<float, AudioUtilities::renderQuantumSize> pannerY;
+    std::array<float, AudioUtilities::renderQuantumSize> pannerZ;
 
-    float orientationX[AudioUtilities::renderQuantumSize];
-    float orientationY[AudioUtilities::renderQuantumSize];
-    float orientationZ[AudioUtilities::renderQuantumSize];
+    std::array<float, AudioUtilities::renderQuantumSize> orientationX;
+    std::array<float, AudioUtilities::renderQuantumSize> orientationY;
+    std::array<float, AudioUtilities::renderQuantumSize> orientationZ;
 
-    m_positionX->calculateSampleAccurateValues(pannerX, framesToProcess);
-    m_positionY->calculateSampleAccurateValues(pannerY, framesToProcess);
-    m_positionZ->calculateSampleAccurateValues(pannerZ, framesToProcess);
-    m_orientationX->calculateSampleAccurateValues(orientationX, framesToProcess);
-    m_orientationY->calculateSampleAccurateValues(orientationY, framesToProcess);
-    m_orientationZ->calculateSampleAccurateValues(orientationZ, framesToProcess);
+    m_positionX->calculateSampleAccurateValues(std::span { pannerX }.first(framesToProcess));
+    m_positionY->calculateSampleAccurateValues(std::span { pannerY }.first(framesToProcess));
+    m_positionZ->calculateSampleAccurateValues(std::span { pannerZ }.first(framesToProcess));
+    m_orientationX->calculateSampleAccurateValues(std::span { orientationX }.first(framesToProcess));
+    m_orientationY->calculateSampleAccurateValues(std::span { orientationY }.first(framesToProcess));
+    m_orientationZ->calculateSampleAccurateValues(std::span { orientationZ }.first(framesToProcess));
 
     // Get the automation values from the listener.
-    const float* listenerX = listener().positionXValues(AudioUtilities::renderQuantumSize);
-    const float* listenerY = listener().positionYValues(AudioUtilities::renderQuantumSize);
-    const float* listenerZ = listener().positionZValues(AudioUtilities::renderQuantumSize);
+    auto listenerX = listener().positionXValues(AudioUtilities::renderQuantumSize);
+    auto listenerY = listener().positionYValues(AudioUtilities::renderQuantumSize);
+    auto listenerZ = listener().positionZValues(AudioUtilities::renderQuantumSize);
 
-    const float* forwardX = listener().forwardXValues(AudioUtilities::renderQuantumSize);
-    const float* forwardY = listener().forwardYValues(AudioUtilities::renderQuantumSize);
-    const float* forwardZ = listener().forwardZValues(AudioUtilities::renderQuantumSize);
+    auto forwardX = listener().forwardXValues(AudioUtilities::renderQuantumSize);
+    auto forwardY = listener().forwardYValues(AudioUtilities::renderQuantumSize);
+    auto forwardZ = listener().forwardZValues(AudioUtilities::renderQuantumSize);
 
-    const float* upX = listener().upXValues(AudioUtilities::renderQuantumSize);
-    const float* upY = listener().upYValues(AudioUtilities::renderQuantumSize);
-    const float* upZ = listener().upZValues(AudioUtilities::renderQuantumSize);
+    auto upX = listener().upXValues(AudioUtilities::renderQuantumSize);
+    auto upY = listener().upYValues(AudioUtilities::renderQuantumSize);
+    auto upZ = listener().upZValues(AudioUtilities::renderQuantumSize);
 
     // Compute the azimuth, elevation, and total gains for each position.
-    double azimuth[AudioUtilities::renderQuantumSize];
-    double elevation[AudioUtilities::renderQuantumSize];
-    float totalGain[AudioUtilities::renderQuantumSize];
+    std::array<double, AudioUtilities::renderQuantumSize> azimuth;
+    std::array<double, AudioUtilities::renderQuantumSize> elevation;
+    std::array<float, AudioUtilities::renderQuantumSize> totalGain;
 
     for (size_t k = 0; k < framesToProcess; ++k) {
         FloatPoint3D pannerPosition(pannerX[k], pannerY[k], pannerZ[k]);
@@ -234,8 +233,8 @@ void PannerNode::processSampleAccurateValues(AudioBus* destination, const AudioB
         totalGain[k] = calculateDistanceConeGain(pannerPosition, orientation, listenerPosition, m_distanceEffect, m_coneEffect);
     }
 
-    m_panner->panWithSampleAccurateValues(azimuth, elevation, source, destination, framesToProcess);
-    destination->copyWithSampleAccurateGainValuesFrom(*destination, totalGain, framesToProcess);
+    m_panner->panWithSampleAccurateValues(std::span { azimuth }, std::span { elevation }, source, destination, framesToProcess);
+    destination.copyWithSampleAccurateGainValuesFrom(destination, std::span { totalGain }.first(framesToProcess));
 }
 
 bool PannerNode::hasSampleAccurateValues() const
@@ -354,7 +353,7 @@ ExceptionOr<void> PannerNode::setRefDistanceForBindings(double refDistance)
     ASSERT(isMainThread());
 
     if (refDistance < 0)
-        return Exception { RangeError, "refDistance cannot be set to a negative value"_s };
+        return Exception { ExceptionCode::RangeError, "refDistance cannot be set to a negative value"_s };
     
     // This synchronizes with process().
     Locker locker { m_processLock };
@@ -372,7 +371,7 @@ ExceptionOr<void> PannerNode::setMaxDistanceForBindings(double maxDistance)
     ASSERT(isMainThread());
 
     if (maxDistance <= 0)
-        return Exception { RangeError, "maxDistance cannot be set to a non-positive value"_s };
+        return Exception { ExceptionCode::RangeError, "maxDistance cannot be set to a non-positive value"_s };
     
     // This synchronizes with process().
     Locker locker { m_processLock };
@@ -390,7 +389,7 @@ ExceptionOr<void> PannerNode::setRolloffFactorForBindings(double rolloffFactor)
     ASSERT(isMainThread());
 
     if (rolloffFactor < 0)
-        return Exception { RangeError, "rolloffFactor cannot be set to a negative value"_s };
+        return Exception { ExceptionCode::RangeError, "rolloffFactor cannot be set to a negative value"_s };
     
     // This synchronizes with process().
     Locker locker { m_processLock };
@@ -408,7 +407,7 @@ ExceptionOr<void> PannerNode::setConeOuterGainForBindings(double gain)
     ASSERT(isMainThread());
 
     if (gain < 0 || gain > 1)
-        return Exception { InvalidStateError, "coneOuterGain must be in [0, 1]"_s };
+        return Exception { ExceptionCode::InvalidStateError, "coneOuterGain must be in [0, 1]"_s };
     
     // This synchronizes with process().
     Locker locker { m_processLock };
@@ -454,7 +453,7 @@ ExceptionOr<void> PannerNode::setChannelCount(unsigned channelCount)
     ASSERT(isMainThread());
 
     if (channelCount > 2)
-        return Exception { NotSupportedError, "PannerNode's channelCount cannot be greater than 2"_s };
+        return Exception { ExceptionCode::NotSupportedError, "PannerNode's channelCount cannot be greater than 2"_s };
     
     return AudioNode::setChannelCount(channelCount);
 }
@@ -464,7 +463,7 @@ ExceptionOr<void> PannerNode::setChannelCountMode(ChannelCountMode mode)
     ASSERT(isMainThread());
 
     if (mode == ChannelCountMode::Max)
-        return Exception { NotSupportedError, "PannerNode's channelCountMode cannot be max"_s };
+        return Exception { ExceptionCode::NotSupportedError, "PannerNode's channelCountMode cannot be max"_s };
     
     return AudioNode::setChannelCountMode(mode);
 }
@@ -510,7 +509,7 @@ auto PannerNode::calculateAzimuthElevation(const FloatPoint3D& position, const F
         azimuth = 450.0 - azimuth;
 
     // Elevation
-    double elevation = 90.0 - 180.0 * acos(sourceListener.dot(up)) / piDouble;
+    double elevation = 90.0 - 180.0 * acos(sourceListener.dot(up)) / std::numbers::pi;
     fixNANs(elevation); // avoid illegal values
 
     if (elevation > 90.0)
