@@ -26,13 +26,14 @@
 #include "AccessibilityAtspiInterfaces.h"
 #include "Document.h"
 #include "FocusController.h"
-#include "Frame.h"
-#include "FrameView.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "Page.h"
 #include <glib/gi18n-lib.h>
 #include <locale.h>
 
 namespace WebCore {
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(AccessibilityRootAtspi);
 
 Ref<AccessibilityRootAtspi> AccessibilityRootAtspi::create(Page& page)
 {
@@ -205,7 +206,11 @@ AccessibilityObjectAtspi* AccessibilityRootAtspi::child() const
     if (!m_page)
         return nullptr;
 
-    Frame& frame = m_page->mainFrame();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+    if (!localMainFrame)
+        return nullptr;
+
+    auto& frame = *localMainFrame;
     if (!frame.document())
         return nullptr;
 
@@ -214,7 +219,7 @@ AccessibilityObjectAtspi* AccessibilityRootAtspi::child() const
     if (!cache)
         return nullptr;
 
-    AXCoreObject* rootObject = cache->rootObject();
+    AXCoreObject* rootObject = cache->rootObjectForFrame(frame);
     return rootObject ? rootObject->wrapper() : nullptr;
 }
 
@@ -266,18 +271,18 @@ GDBusInterfaceVTable AccessibilityRootAtspi::s_componentFunctions = {
         else if (!g_strcmp0(methodName, "GetExtents")) {
             uint32_t coordinateType;
             g_variant_get(parameters, "(u)", &coordinateType);
-            auto rect = rootObject.frameRect(coordinateType);
+            auto rect = rootObject.frameRect(static_cast<Atspi::CoordinateType>(coordinateType));
             g_dbus_method_invocation_return_value(invocation, g_variant_new("((iiii))", rect.x(), rect.y(), rect.width(), rect.height()));
         } else if (!g_strcmp0(methodName, "GetPosition")) {
             uint32_t coordinateType;
             g_variant_get(parameters, "(u)", &coordinateType);
-            auto rect = rootObject.frameRect(coordinateType);
+            auto rect = rootObject.frameRect(static_cast<Atspi::CoordinateType>(coordinateType));
             g_dbus_method_invocation_return_value(invocation, g_variant_new("((ii))", rect.x(), rect.y()));
         } else if (!g_strcmp0(methodName, "GetSize")) {
             auto rect = rootObject.frameRect(Atspi::CoordinateType::ParentCoordinates);
             g_dbus_method_invocation_return_value(invocation, g_variant_new("((ii))", rect.width(), rect.height()));
         } else if (!g_strcmp0(methodName, "GetLayer"))
-            g_dbus_method_invocation_return_value(invocation, g_variant_new("(u)", Atspi::ComponentLayer::WidgetLayer));
+            g_dbus_method_invocation_return_value(invocation, g_variant_new("(u)", static_cast<uint32_t>(Atspi::ComponentLayer::WidgetLayer)));
         else if (!g_strcmp0(methodName, "GetMDIZOrder"))
             g_dbus_method_invocation_return_value(invocation, g_variant_new("(n)", 0));
         else if (!g_strcmp0(methodName, "GrabFocus"))
@@ -295,12 +300,16 @@ GDBusInterfaceVTable AccessibilityRootAtspi::s_componentFunctions = {
     { nullptr }
 };
 
-IntRect AccessibilityRootAtspi::frameRect(uint32_t coordinateType) const
+IntRect AccessibilityRootAtspi::frameRect(Atspi::CoordinateType coordinateType) const
 {
     if (!m_page)
         return { };
 
-    auto* frameView = m_page->mainFrame().view();
+    auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->mainFrame());
+    if (!localMainFrame)
+        return { };
+
+    auto* frameView = localMainFrame->view();
     if (!frameView)
         return { };
 
