@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Apple Inc. All Rights Reserved.
+ * Copyright (C) 2016 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,7 @@
 #include "StructuredClone.h"
 
 #include "JSDOMBinding.h"
+#include "JSDOMConvertBufferSource.h"
 #include "JSDOMExceptionHandling.h"
 #include <JavaScriptCore/JSTypedArrays.h>
 
@@ -59,7 +60,7 @@ static EncodedJSValue cloneArrayBufferImpl(JSGlobalObject* lexicalGlobalObject,
         int srcLength = static_cast<int>(callFrame->uncheckedArgument(2).toNumber(lexicalGlobalObject));
         return JSValue::encode(JSArrayBuffer::create(lexicalGlobalObject->vm(), lexicalGlobalObject->arrayBufferStructure(ArrayBufferSharingMode::Default), buffer->slice(srcByteOffset, srcByteOffset + srcLength)));
     }
-    return JSValue::encode(JSArrayBuffer::create(lexicalGlobalObject->vm(), lexicalGlobalObject->arrayBufferStructure(ArrayBufferSharingMode::Default), ArrayBuffer::tryCreate(buffer->data(), buffer->byteLength())));
+    return JSValue::encode(JSArrayBuffer::create(lexicalGlobalObject->vm(), lexicalGlobalObject->arrayBufferStructure(ArrayBufferSharingMode::Default), ArrayBuffer::tryCreate(buffer->span())));
 }
 
 JSC_DEFINE_HOST_FUNCTION(cloneArrayBuffer, (JSGlobalObject* globalObject, CallFrame* callFrame))
@@ -77,33 +78,70 @@ JSC_DEFINE_HOST_FUNCTION(structuredCloneForStream, (JSGlobalObject* globalObject
 
     JSValue value = callFrame->uncheckedArgument(0);
 
-    if (value.inherits<JSArrayBuffer>())
-        RELEASE_AND_RETURN(scope, cloneArrayBufferImpl(globalObject, callFrame, CloneMode::Full));
+    auto cloneArrayBuffer = [&](ArrayBuffer& buffer) -> RefPtr<ArrayBuffer> {
+        auto scope = DECLARE_THROW_SCOPE(vm);
+        size_t byteLength = buffer.byteLength();
+        auto result = ArrayBuffer::tryCreate(byteLength, 1, buffer.maxByteLength());
+        if (!result) [[unlikely]] {
+            throwOutOfMemoryError(globalObject, scope);
+            return nullptr;
+        }
+
+        memcpySpan(result->mutableSpan(), buffer.span().first(byteLength));
+        return result;
+    };
+
+    if (value.inherits<JSArrayBuffer>()) {
+        auto* buffer = toUnsharedArrayBuffer(vm, value);
+        if (!buffer || buffer->isDetached()) [[unlikely]] {
+            throwDataCloneError(*globalObject, scope);
+            return { };
+        }
+
+        auto result = cloneArrayBuffer(*buffer);
+        RETURN_IF_EXCEPTION(scope, { });
+
+        return JSValue::encode(JSArrayBuffer::create(globalObject->vm(), globalObject->arrayBufferStructure(ArrayBufferSharingMode::Default), result.releaseNonNull()));
+    }
 
     if (value.inherits<JSArrayBufferView>()) {
         auto* bufferView = jsCast<JSArrayBufferView*>(value);
         ASSERT(bufferView);
 
         auto* buffer = bufferView->unsharedBuffer();
-        if (!buffer) {
+        if (!buffer || buffer->isDetached()) [[unlikely]] {
             throwDataCloneError(*globalObject, scope);
             return { };
         }
-        auto bufferClone = ArrayBuffer::tryCreate(buffer->data(), buffer->byteLength());
-        Structure* structure = bufferView->structure();
 
+        auto bufferClone = cloneArrayBuffer(*buffer);
+        RETURN_IF_EXCEPTION(scope, { });
+
+        size_t byteOffset = 0;
+        std::optional<size_t> length;
+        if (bufferView->isResizableOrGrowableShared()) {
+            byteOffset = bufferView->byteOffsetRaw();
+            if (!bufferView->isAutoLength())
+                length = bufferView->lengthRaw();
+        } else {
+            byteOffset = bufferView->byteOffset();
+            length = bufferView->length();
+        }
+
+        switch (typedArrayType(bufferView->type())) {
 #define CLONE_TYPED_ARRAY(name) \
-        do { \
-            if (bufferView->inherits<JS##name##Array>()) \
-                RELEASE_AND_RETURN(scope, JSValue::encode(JS##name##Array::create(globalObject, structure, WTFMove(bufferClone), bufferView->byteOffset(), bufferView->length()))); \
-        } while (0);
-
+        case Type##name: { \
+            RELEASE_AND_RETURN(scope, JSValue::encode(toJS(globalObject, globalObject, name##Array::wrappedAs(bufferClone.releaseNonNull(), byteOffset, length).get()))); \
+        }
         FOR_EACH_TYPED_ARRAY_TYPE_EXCLUDING_DATA_VIEW(CLONE_TYPED_ARRAY)
-
 #undef CLONE_TYPED_ARRAY
-
-        if (value.inherits<JSDataView>())
-            RELEASE_AND_RETURN(scope, JSValue::encode(JSDataView::create(globalObject, structure, WTFMove(bufferClone), bufferView->byteOffset(), bufferView->length())));
+        case TypeDataView: {
+            RELEASE_AND_RETURN(scope, JSValue::encode(toJS(globalObject, globalObject, DataView::wrappedAs(bufferClone.releaseNonNull(), byteOffset, length).get())));
+        }
+        default:
+            RELEASE_ASSERT_NOT_REACHED();
+            break;
+        }
     }
 
     throwTypeError(globalObject, scope, "structuredClone not implemented for non-ArrayBuffer / non-ArrayBufferView"_s);
