@@ -1,5 +1,5 @@
 // Copyright 2015 The Chromium Authors. All rights reserved.
-// Copyright (C) 2016 Apple Inc. All rights reserved.
+// Copyright (C) 2016-2024 Apple Inc. All rights reserved.
 //
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
@@ -34,62 +34,99 @@
 #include "CSSParserObserverWrapper.h"
 #include "CSSParserTokenRange.h"
 #include "CSSTokenizerInputStream.h"
-#include "HTMLParserIdioms.h"
 #include <wtf/text/StringBuilder.h>
+#include <wtf/text/StringToIntegerConversion.h>
 #include <wtf/unicode/CharacterNames.h>
 
 namespace WebCore {
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(CSSTokenizer);
+
+// https://drafts.csswg.org/css-syntax/#input-preprocessing
+String CSSTokenizer::preprocessString(const String& string)
+{
+    // We don't replace '\r' and '\f' with '\n' as the specification suggests, instead
+    // we treat them all the same in the isNewline function below.
+    StringImpl* oldImpl = string.impl();
+    String replaced = makeStringByReplacingAll(string, '\0', replacementCharacter);
+    replaced = replaceUnpairedSurrogatesWithReplacementCharacter(WTFMove(replaced));
+    if (replaced.impl() != oldImpl)
+        registerString(replaced);
+    return replaced;
+}
+
+std::unique_ptr<CSSTokenizer> CSSTokenizer::tryCreate(const String& string)
+{
+    bool success = true;
+    // We can't use makeUnique here because it does not have access to this private constructor.
+    auto tokenizer = std::unique_ptr<CSSTokenizer>(new CSSTokenizer(string, nullptr, &success));
+    if (!success) [[unlikely]]
+        return nullptr;
+    return tokenizer;
+}
+
+std::unique_ptr<CSSTokenizer> CSSTokenizer::tryCreate(const String& string, CSSParserObserverWrapper& wrapper)
+{
+    bool success = true;
+    // We can't use makeUnique here because it does not have access to this private constructor.
+    auto tokenizer = std::unique_ptr<CSSTokenizer>(new CSSTokenizer(string, &wrapper, &success));
+    if (!success) [[unlikely]]
+        return nullptr;
+    return tokenizer;
+}
 
 CSSTokenizer::CSSTokenizer(const String& string)
-    : m_input(string)
+    : CSSTokenizer(string, nullptr, nullptr)
 {
-    // According to the spec, we should perform preprocessing here.
-    // See: http://dev.w3.org/csswg/css-syntax/#input-preprocessing
-    //
-    // However, we can skip this step since:
-    // * We're using HTML spaces (which accept \r and \f as a valid white space)
-    // * Do not count white spaces
-    // * CSSTokenizerInputStream::nextInputChar() replaces NULLs for replacement characters
+}
+
+CSSTokenizer::CSSTokenizer(const String& string, CSSParserObserverWrapper& wrapper)
+    : CSSTokenizer(string, &wrapper, nullptr)
+{
+}
+
+CSSTokenizer::CSSTokenizer(const String& string, CSSParserObserverWrapper* wrapper, bool* constructionSuccessPtr)
+    : m_input(preprocessString(string))
+{
+    if (constructionSuccessPtr)
+        *constructionSuccessPtr = true;
 
     if (string.isEmpty())
         return;
 
     // To avoid resizing we err on the side of reserving too much space.
     // Most strings we tokenize have about 3.5 to 5 characters per token.
-    m_tokens.reserveInitialCapacity(string.length() / 3);
-
-    while (true) {
-        CSSParserToken token = nextToken();
-        if (token.type() == CommentToken)
-            continue;
-        if (token.type() == EOFToken)
-            return;
-        m_tokens.append(token);
-    }
-}
-
-CSSTokenizer::CSSTokenizer(const String& string, CSSParserObserverWrapper& wrapper)
-    : m_input(string)
-{
-    if (string.isEmpty())
+    if (!m_tokens.tryReserveInitialCapacity(string.length() / 3)) [[unlikely]] {
+        // When constructionSuccessPtr is null, our policy is to crash on failure.
+        RELEASE_ASSERT(constructionSuccessPtr);
+        *constructionSuccessPtr = false;
         return;
+    }
 
     unsigned offset = 0;
     while (true) {
         CSSParserToken token = nextToken();
         if (token.type() == EOFToken)
             break;
-        if (token.type() == CommentToken)
-            wrapper.addComment(offset, m_input.offset(), m_tokens.size());
-        else {
-            m_tokens.append(token);
-            wrapper.addToken(offset);
+        if (token.type() == CommentToken) {
+            if (wrapper)
+                wrapper->addComment(offset, m_input.offset(), m_tokens.size());
+        } else {
+            if (!m_tokens.tryAppend(token)) [[unlikely]] {
+                // When constructionSuccessPtr is null, our policy is to crash on failure.
+                RELEASE_ASSERT(constructionSuccessPtr);
+                *constructionSuccessPtr = false;
+                return;
+            }
+            if (wrapper)
+                wrapper->addToken(offset);
         }
         offset = m_input.offset();
     }
 
-    wrapper.addToken(offset);
-    wrapper.finalizeConstruction(m_tokens.begin());
+    if (wrapper) {
+        wrapper->addToken(offset);
+        wrapper->finalizeConstruction(m_tokens.begin());
+    }
 }
 
 CSSParserTokenRange CSSTokenizer::tokenRange() const
@@ -102,34 +139,46 @@ unsigned CSSTokenizer::tokenCount()
     return m_tokens.size();
 }
 
-static bool isNewLine(UChar cc)
+bool CSSTokenizer::isWhitespace(CSSParserTokenType type)
+{
+    return type == NonNewlineWhitespaceToken || type == NewlineToken;
+}
+
+bool CSSTokenizer::isNewline(char16_t cc)
 {
     // We check \r and \f here, since we have no preprocessing stage
     return (cc == '\r' || cc == '\n' || cc == '\f');
 }
 
-// http://dev.w3.org/csswg/css-syntax/#check-if-two-code-points-are-a-valid-escape
-static bool twoCharsAreValidEscape(UChar first, UChar second)
+CSSParserToken CSSTokenizer::newline(char16_t)
 {
-    return first == '\\' && !isNewLine(second);
+    return CSSParserToken(NewlineToken);
 }
 
-void CSSTokenizer::reconsume(UChar c)
+// http://dev.w3.org/csswg/css-syntax/#check-if-two-code-points-are-a-valid-escape
+static bool twoCharsAreValidEscape(char16_t first, char16_t second)
+{
+    return first == '\\' && !CSSTokenizer::isNewline(second);
+}
+
+void CSSTokenizer::reconsume(char16_t c)
 {
     m_input.pushBack(c);
 }
 
-UChar CSSTokenizer::consume()
+char16_t CSSTokenizer::consume()
 {
-    UChar current = m_input.nextInputChar();
+    char16_t current = m_input.nextInputChar();
     m_input.advance();
     return current;
 }
 
-CSSParserToken CSSTokenizer::whiteSpace(UChar /*cc*/)
+CSSParserToken CSSTokenizer::whitespace(char16_t)
 {
-    m_input.advanceUntilNonWhitespace();
-    return CSSParserToken(WhitespaceToken);
+    auto startOffset = m_input.offset();
+    m_input.advanceUntilNewlineOrNonWhitespace();
+    auto whitespaceCount = 1 + (m_input.offset() - startOffset);
+    return CSSParserToken(whitespaceCount);
 }
 
 CSSParserToken CSSTokenizer::blockStart(CSSParserTokenType type)
@@ -153,37 +202,37 @@ CSSParserToken CSSTokenizer::blockEnd(CSSParserTokenType type, CSSParserTokenTyp
     return CSSParserToken(type);
 }
 
-CSSParserToken CSSTokenizer::leftParenthesis(UChar /*cc*/)
+CSSParserToken CSSTokenizer::leftParenthesis(char16_t)
 {
     return blockStart(LeftParenthesisToken);
 }
 
-CSSParserToken CSSTokenizer::rightParenthesis(UChar /*cc*/)
+CSSParserToken CSSTokenizer::rightParenthesis(char16_t)
 {
     return blockEnd(RightParenthesisToken, LeftParenthesisToken);
 }
 
-CSSParserToken CSSTokenizer::leftBracket(UChar /*cc*/)
+CSSParserToken CSSTokenizer::leftBracket(char16_t)
 {
     return blockStart(LeftBracketToken);
 }
 
-CSSParserToken CSSTokenizer::rightBracket(UChar /*cc*/)
+CSSParserToken CSSTokenizer::rightBracket(char16_t)
 {
     return blockEnd(RightBracketToken, LeftBracketToken);
 }
 
-CSSParserToken CSSTokenizer::leftBrace(UChar /*cc*/)
+CSSParserToken CSSTokenizer::leftBrace(char16_t)
 {
     return blockStart(LeftBraceToken);
 }
 
-CSSParserToken CSSTokenizer::rightBrace(UChar /*cc*/)
+CSSParserToken CSSTokenizer::rightBrace(char16_t)
 {
     return blockEnd(RightBraceToken, LeftBraceToken);
 }
 
-CSSParserToken CSSTokenizer::plusOrFullStop(UChar cc)
+CSSParserToken CSSTokenizer::plusOrFullStop(char16_t cc)
 {
     if (nextCharsAreNumber(cc)) {
         reconsume(cc);
@@ -192,7 +241,7 @@ CSSParserToken CSSTokenizer::plusOrFullStop(UChar cc)
     return CSSParserToken(DelimiterToken, cc);
 }
 
-CSSParserToken CSSTokenizer::asterisk(UChar cc)
+CSSParserToken CSSTokenizer::asterisk(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '*');
     if (consumeIfNext('='))
@@ -200,31 +249,28 @@ CSSParserToken CSSTokenizer::asterisk(UChar cc)
     return CSSParserToken(DelimiterToken, '*');
 }
 
-CSSParserToken CSSTokenizer::lessThan(UChar cc)
+CSSParserToken CSSTokenizer::lessThan(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '<');
-    if (m_input.peekWithoutReplacement(0) == '!'
-        && m_input.peekWithoutReplacement(1) == '-'
-        && m_input.peekWithoutReplacement(2) == '-') {
+    if (m_input.peek(0) == '!' && m_input.peek(1) == '-' && m_input.peek(2) == '-') {
         m_input.advance(3);
         return CSSParserToken(CDOToken);
     }
     return CSSParserToken(DelimiterToken, '<');
 }
 
-CSSParserToken CSSTokenizer::comma(UChar /*cc*/)
+CSSParserToken CSSTokenizer::comma(char16_t)
 {
     return CSSParserToken(CommaToken);
 }
 
-CSSParserToken CSSTokenizer::hyphenMinus(UChar cc)
+CSSParserToken CSSTokenizer::hyphenMinus(char16_t cc)
 {
     if (nextCharsAreNumber(cc)) {
         reconsume(cc);
         return consumeNumericToken();
     }
-    if (m_input.peekWithoutReplacement(0) == '-'
-        && m_input.peekWithoutReplacement(1) == '>') {
+    if (m_input.peek(0) == '-' && m_input.peek(1) == '>') {
         m_input.advance(2);
         return CSSParserToken(CDCToken);
     }
@@ -235,7 +281,7 @@ CSSParserToken CSSTokenizer::hyphenMinus(UChar cc)
     return CSSParserToken(DelimiterToken, cc);
 }
 
-CSSParserToken CSSTokenizer::solidus(UChar cc)
+CSSParserToken CSSTokenizer::solidus(char16_t cc)
 {
     if (consumeIfNext('*')) {
         // These get ignored, but we need a value to return.
@@ -246,20 +292,20 @@ CSSParserToken CSSTokenizer::solidus(UChar cc)
     return CSSParserToken(DelimiterToken, cc);
 }
 
-CSSParserToken CSSTokenizer::colon(UChar /*cc*/)
+CSSParserToken CSSTokenizer::colon(char16_t)
 {
     return CSSParserToken(ColonToken);
 }
 
-CSSParserToken CSSTokenizer::semiColon(UChar /*cc*/)
+CSSParserToken CSSTokenizer::semiColon(char16_t)
 {
     return CSSParserToken(SemicolonToken);
 }
 
-CSSParserToken CSSTokenizer::hash(UChar cc)
+CSSParserToken CSSTokenizer::hash(char16_t cc)
 {
-    UChar nextChar = m_input.peekWithoutReplacement(0);
-    if (isNameCodePoint(nextChar) || twoCharsAreValidEscape(nextChar, m_input.peekWithoutReplacement(1))) {
+    char16_t nextChar = m_input.peek(0);
+    if (isNameCodePoint(nextChar) || twoCharsAreValidEscape(nextChar, m_input.peek(1))) {
         HashTokenType type = nextCharsAreIdentifier() ? HashTokenId : HashTokenUnrestricted;
         return CSSParserToken(type, consumeName());
     }
@@ -267,7 +313,7 @@ CSSParserToken CSSTokenizer::hash(UChar cc)
     return CSSParserToken(DelimiterToken, cc);
 }
 
-CSSParserToken CSSTokenizer::circumflexAccent(UChar cc)
+CSSParserToken CSSTokenizer::circumflexAccent(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '^');
     if (consumeIfNext('='))
@@ -275,7 +321,7 @@ CSSParserToken CSSTokenizer::circumflexAccent(UChar cc)
     return CSSParserToken(DelimiterToken, '^');
 }
 
-CSSParserToken CSSTokenizer::dollarSign(UChar cc)
+CSSParserToken CSSTokenizer::dollarSign(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '$');
     if (consumeIfNext('='))
@@ -283,7 +329,7 @@ CSSParserToken CSSTokenizer::dollarSign(UChar cc)
     return CSSParserToken(DelimiterToken, '$');
 }
 
-CSSParserToken CSSTokenizer::verticalLine(UChar cc)
+CSSParserToken CSSTokenizer::verticalLine(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '|');
     if (consumeIfNext('='))
@@ -293,7 +339,7 @@ CSSParserToken CSSTokenizer::verticalLine(UChar cc)
     return CSSParserToken(DelimiterToken, '|');
 }
 
-CSSParserToken CSSTokenizer::tilde(UChar cc)
+CSSParserToken CSSTokenizer::tilde(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '~');
     if (consumeIfNext('='))
@@ -301,7 +347,7 @@ CSSParserToken CSSTokenizer::tilde(UChar cc)
     return CSSParserToken(DelimiterToken, '~');
 }
 
-CSSParserToken CSSTokenizer::commercialAt(UChar cc)
+CSSParserToken CSSTokenizer::commercialAt(char16_t cc)
 {
     ASSERT_UNUSED(cc, cc == '@');
     if (nextCharsAreIdentifier())
@@ -309,49 +355,38 @@ CSSParserToken CSSTokenizer::commercialAt(UChar cc)
     return CSSParserToken(DelimiterToken, '@');
 }
 
-CSSParserToken CSSTokenizer::reverseSolidus(UChar cc)
+CSSParserToken CSSTokenizer::reverseSolidus(char16_t cc)
 {
-    if (twoCharsAreValidEscape(cc, m_input.peekWithoutReplacement(0))) {
+    if (twoCharsAreValidEscape(cc, m_input.peek(0))) {
         reconsume(cc);
         return consumeIdentLikeToken();
     }
     return CSSParserToken(DelimiterToken, cc);
 }
 
-CSSParserToken CSSTokenizer::asciiDigit(UChar cc)
+CSSParserToken CSSTokenizer::asciiDigit(char16_t cc)
 {
     reconsume(cc);
     return consumeNumericToken();
 }
 
-CSSParserToken CSSTokenizer::letterU(UChar cc)
-{
-    if (m_input.peekWithoutReplacement(0) == '+'
-        && (isASCIIHexDigit(m_input.peekWithoutReplacement(1)) || m_input.peekWithoutReplacement(1) == '?')) {
-        m_input.advance();
-        return consumeUnicodeRange();
-    }
-    reconsume(cc);
-    return consumeIdentLikeToken();
-}
-
-CSSParserToken CSSTokenizer::nameStart(UChar cc)
+CSSParserToken CSSTokenizer::nameStart(char16_t cc)
 {
     reconsume(cc);
     return consumeIdentLikeToken();
 }
 
-CSSParserToken CSSTokenizer::stringStart(UChar cc)
+CSSParserToken CSSTokenizer::stringStart(char16_t cc)
 {
     return consumeStringTokenUntil(cc);
 }
 
-CSSParserToken CSSTokenizer::endOfFile(UChar /*cc*/)
+CSSParserToken CSSTokenizer::endOfFile(char16_t)
 {
     return CSSParserToken(EOFToken);
 }
 
-const CSSTokenizer::CodePoint CSSTokenizer::codePoints[128] = {
+const std::array<CSSTokenizer::CodePoint, 128> CSSTokenizer::codePoints {
     &CSSTokenizer::endOfFile,
     0,
     0,
@@ -361,11 +396,11 @@ const CSSTokenizer::CodePoint CSSTokenizer::codePoints[128] = {
     0,
     0,
     0,
-    &CSSTokenizer::whiteSpace,
-    &CSSTokenizer::whiteSpace,
+    &CSSTokenizer::whitespace,
+    &CSSTokenizer::newline, // '\n'
     0,
-    &CSSTokenizer::whiteSpace,
-    &CSSTokenizer::whiteSpace,
+    &CSSTokenizer::newline, // '\f'
+    &CSSTokenizer::newline, // '\r'
     0,
     0,
     0,
@@ -384,7 +419,7 @@ const CSSTokenizer::CodePoint CSSTokenizer::codePoints[128] = {
     0,
     0,
     0,
-    &CSSTokenizer::whiteSpace,
+    &CSSTokenizer::whitespace,
     0,
     &CSSTokenizer::stringStart,
     &CSSTokenizer::hash,
@@ -437,7 +472,7 @@ const CSSTokenizer::CodePoint CSSTokenizer::codePoints[128] = {
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
-    &CSSTokenizer::letterU,
+    &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
@@ -469,7 +504,7 @@ const CSSTokenizer::CodePoint CSSTokenizer::codePoints[128] = {
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
-    &CSSTokenizer::letterU,
+    &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
     &CSSTokenizer::nameStart,
@@ -494,7 +529,7 @@ CSSParserToken CSSTokenizer::nextToken()
     // State-machine tokenizers are easier to write to handle
     // incremental tokenization of partial sources.
     // However, for now we follow the spec exactly.
-    UChar cc = consume();
+    char16_t cc = consume();
     CodePoint codePointFunc = 0;
 
     if (isASCII(cc)) {
@@ -515,11 +550,13 @@ CSSParserToken CSSTokenizer::consumeNumber()
 {
     ASSERT(nextCharsAreNumber());
 
+    auto startOffset = m_input.offset();
+
     NumericValueType type = IntegerValueType;
     NumericSign sign = NoSign;
     unsigned numberLength = 0;
 
-    UChar next = m_input.peekWithoutReplacement(0);
+    char16_t next = m_input.peek(0);
     if (next == '+') {
         ++numberLength;
         sign = PlusSign;
@@ -529,19 +566,19 @@ CSSParserToken CSSTokenizer::consumeNumber()
     }
 
     numberLength = m_input.skipWhilePredicate<isASCIIDigit>(numberLength);
-    next = m_input.peekWithoutReplacement(numberLength);
-    if (next == '.' && isASCIIDigit(m_input.peekWithoutReplacement(numberLength + 1))) {
+    next = m_input.peek(numberLength);
+    if (next == '.' && isASCIIDigit(m_input.peek(numberLength + 1))) {
         type = NumberValueType;
         numberLength = m_input.skipWhilePredicate<isASCIIDigit>(numberLength + 2);
-        next = m_input.peekWithoutReplacement(numberLength);
+        next = m_input.peek(numberLength);
     }
 
     if (next == 'E' || next == 'e') {
-        next = m_input.peekWithoutReplacement(numberLength + 1);
+        next = m_input.peek(numberLength + 1);
         if (isASCIIDigit(next)) {
             type = NumberValueType;
             numberLength = m_input.skipWhilePredicate<isASCIIDigit>(numberLength + 1);
-        } else if ((next == '+' || next == '-') && isASCIIDigit(m_input.peekWithoutReplacement(numberLength + 2))) {
+        } else if ((next == '+' || next == '-') && isASCIIDigit(m_input.peek(numberLength + 2))) {
             type = NumberValueType;
             numberLength = m_input.skipWhilePredicate<isASCIIDigit>(numberLength + 3);
         }
@@ -550,7 +587,7 @@ CSSParserToken CSSTokenizer::consumeNumber()
     double value = m_input.getDouble(0, numberLength);
     m_input.advance(numberLength);
 
-    return CSSParserToken(NumberToken, value, type, sign);
+    return CSSParserToken(value, type, sign, m_input.rangeAt(startOffset, m_input.offset() - startOffset));
 }
 
 // http://www.w3.org/TR/css3-syntax/#consume-a-numeric-token
@@ -569,13 +606,13 @@ CSSParserToken CSSTokenizer::consumeIdentLikeToken()
 {
     StringView name = consumeName();
     if (consumeIfNext('(')) {
-        if (equalIgnoringASCIICase(name, "url")) {
+        if (equalLettersIgnoringASCIICase(name, "url"_s)) {
             // The spec is slightly different so as to avoid dropping whitespace
             // tokens, but they wouldn't be used and this is easier.
             m_input.advanceUntilNonWhitespace();
-            UChar next = m_input.peekWithoutReplacement(0);
+            char16_t next = m_input.peek(0);
             if (next != '"' && next != '\'')
-                return consumeUrlToken();
+                return consumeURLToken();
         }
         return blockStart(LeftParenthesisToken, FunctionToken, name);
     }
@@ -583,90 +620,59 @@ CSSParserToken CSSTokenizer::consumeIdentLikeToken()
 }
 
 // http://dev.w3.org/csswg/css-syntax/#consume-a-string-token
-CSSParserToken CSSTokenizer::consumeStringTokenUntil(UChar endingCodePoint)
+CSSParserToken CSSTokenizer::consumeStringTokenUntil(char16_t endingCodePoint)
 {
     // Strings without escapes get handled without allocations
     for (unsigned size = 0; ; size++) {
-        UChar cc = m_input.peekWithoutReplacement(size);
+        char16_t cc = m_input.peek(size);
         if (cc == endingCodePoint) {
             unsigned startOffset = m_input.offset();
             m_input.advance(size + 1);
             return CSSParserToken(StringToken, m_input.rangeAt(startOffset, size));
         }
-        if (isNewLine(cc)) {
+        if (isNewline(cc)) {
             m_input.advance(size);
             return CSSParserToken(BadStringToken);
         }
-        if (cc == '\0' || cc == '\\')
+        if (cc == kEndOfFileMarker || cc == '\\')
             break;
     }
 
     StringBuilder output;
     while (true) {
-        UChar cc = consume();
+        char16_t cc = consume();
         if (cc == endingCodePoint || cc == kEndOfFileMarker)
             return CSSParserToken(StringToken, registerString(output.toString()));
-        if (isNewLine(cc)) {
+        if (isNewline(cc)) {
             reconsume(cc);
             return CSSParserToken(BadStringToken);
         }
         if (cc == '\\') {
             if (m_input.nextInputChar() == kEndOfFileMarker)
                 continue;
-            if (isNewLine(m_input.peekWithoutReplacement(0)))
+            if (isNewline(m_input.peek(0)))
                 consumeSingleWhitespaceIfNext(); // This handles \r\n for us
             else
-                output.appendCharacter(consumeEscape());
+                output.append(consumeEscape());
         } else
             output.append(cc);
     }
 }
 
-CSSParserToken CSSTokenizer::consumeUnicodeRange()
-{
-    ASSERT(isASCIIHexDigit(m_input.peekWithoutReplacement(0)) || m_input.peekWithoutReplacement(0) == '?');
-    int lengthRemaining = 6;
-    UChar32 start = 0;
-
-    while (lengthRemaining && isASCIIHexDigit(m_input.peekWithoutReplacement(0))) {
-        start = start * 16 + toASCIIHexValue(consume());
-        --lengthRemaining;
-    }
-
-    UChar32 end = start;
-    if (lengthRemaining && consumeIfNext('?')) {
-        do {
-            start *= 16;
-            end = end * 16 + 0xF;
-            --lengthRemaining;
-        } while (lengthRemaining && consumeIfNext('?'));
-    } else if (m_input.peekWithoutReplacement(0) == '-' && isASCIIHexDigit(m_input.peekWithoutReplacement(1))) {
-        m_input.advance();
-        lengthRemaining = 6;
-        end = 0;
-        do {
-            end = end * 16 + toASCIIHexValue(consume());
-            --lengthRemaining;
-        } while (lengthRemaining && isASCIIHexDigit(m_input.peekWithoutReplacement(0)));
-    }
-
-    return CSSParserToken(UnicodeRangeToken, start, end);
-}
-
 // http://dev.w3.org/csswg/css-syntax/#non-printable-code-point
-static bool isNonPrintableCodePoint(UChar cc)
+static bool isNonPrintableCodePoint(char16_t cc)
 {
     return cc <= '\x8' || cc == '\xb' || (cc >= '\xe' && cc <= '\x1f') || cc == '\x7f';
 }
 
 // http://dev.w3.org/csswg/css-syntax/#consume-url-token
-CSSParserToken CSSTokenizer::consumeUrlToken()
+CSSParserToken CSSTokenizer::consumeURLToken()
 {
     m_input.advanceUntilNonWhitespace();
 
     // URL tokens without escapes get handled without allocations
     for (unsigned size = 0; ; size++) {
-        UChar cc = m_input.peekWithoutReplacement(size);
+        char16_t cc = m_input.peek(size);
         if (cc == ')') {
             unsigned startOffset = m_input.offset();
             m_input.advance(size + 1);
@@ -678,11 +684,11 @@ CSSParserToken CSSTokenizer::consumeUrlToken()
 
     StringBuilder result;
     while (true) {
-        UChar cc = consume();
+        char16_t cc = consume();
         if (cc == ')' || cc == kEndOfFileMarker)
             return CSSParserToken(UrlToken, registerString(result.toString()));
 
-        if (isHTMLSpace(cc)) {
+        if (isASCIIWhitespace(cc)) {
             m_input.advanceUntilNonWhitespace();
             if (consumeIfNext(')') || m_input.nextInputChar() == kEndOfFileMarker)
                 return CSSParserToken(UrlToken, registerString(result.toString()));
@@ -693,8 +699,8 @@ CSSParserToken CSSTokenizer::consumeUrlToken()
             break;
 
         if (cc == '\\') {
-            if (twoCharsAreValidEscape(cc, m_input.peekWithoutReplacement(0))) {
-                result.appendCharacter(consumeEscape());
+            if (twoCharsAreValidEscape(cc, m_input.peek(0))) {
+                result.append(consumeEscape());
                 continue;
             }
             break;
@@ -711,27 +717,27 @@ CSSParserToken CSSTokenizer::consumeUrlToken()
 void CSSTokenizer::consumeBadUrlRemnants()
 {
     while (true) {
-        UChar cc = consume();
+        char16_t cc = consume();
         if (cc == ')' || cc == kEndOfFileMarker)
             return;
-        if (twoCharsAreValidEscape(cc, m_input.peekWithoutReplacement(0)))
+        if (twoCharsAreValidEscape(cc, m_input.peek(0)))
             consumeEscape();
     }
 }
 
 void CSSTokenizer::consumeSingleWhitespaceIfNext()
 {
-    // We check for \r\n and HTML spaces since we don't do preprocessing
-    UChar next = m_input.peekWithoutReplacement(0);
-    if (next == '\r' && m_input.peekWithoutReplacement(1) == '\n')
+    // We check for \r\n and ASCII whitespace since we don't do preprocessing
+    char16_t next = m_input.peek(0);
+    if (next == '\r' && m_input.peek(1) == '\n')
         m_input.advance(2);
-    else if (isHTMLSpace(next))
+    else if (isASCIIWhitespace(next))
         m_input.advance();
 }
 
 void CSSTokenizer::consumeUntilCommentEndFound()
 {
-    UChar c = consume();
+    char16_t c = consume();
     while (true) {
         if (c == kEndOfFileMarker)
             return;
@@ -745,13 +751,13 @@ void CSSTokenizer::consumeUntilCommentEndFound()
     }
 }
 
-bool CSSTokenizer::consumeIfNext(UChar character)
+bool CSSTokenizer::consumeIfNext(char16_t character)
 {
     // Since we're not doing replacement we can't tell the difference from
     // a NUL in the middle and the kEndOfFileMarker, so character must not be
     // NUL.
     ASSERT(character);
-    if (m_input.peekWithoutReplacement(0) == character) {
+    if (m_input.peek(0) == character) {
         m_input.advance();
         return true;
     }
@@ -763,13 +769,13 @@ StringView CSSTokenizer::consumeName()
 {
     // Names without escapes get handled without allocations
     for (unsigned size = 0; ; ++size) {
-        UChar cc = m_input.peekWithoutReplacement(size);
+        char16_t cc = m_input.peek(size);
         if (isNameCodePoint(cc))
             continue;
-        // peekWithoutReplacement will return NUL when we hit the end of the
+        // peek will return NUL when we hit the end of the
         // input. In that case we want to still use the rangeAt() fast path
         // below.
-        if (cc == '\0' && m_input.offset() + size < m_input.length())
+        if (cc == kEndOfFileMarker && m_input.offset() + size < m_input.length())
             break;
         if (cc == '\\')
             break;
@@ -780,13 +786,13 @@ StringView CSSTokenizer::consumeName()
 
     StringBuilder result;
     while (true) {
-        UChar cc = consume();
+        char16_t cc = consume();
         if (isNameCodePoint(cc)) {
             result.append(cc);
             continue;
         }
-        if (twoCharsAreValidEscape(cc, m_input.peekWithoutReplacement(0))) {
-            result.appendCharacter(consumeEscape());
+        if (twoCharsAreValidEscape(cc, m_input.peek(0))) {
+            result.append(consumeEscape());
             continue;
         }
         reconsume(cc);
@@ -795,24 +801,22 @@ StringView CSSTokenizer::consumeName()
 }
 
 // http://dev.w3.org/csswg/css-syntax/#consume-an-escaped-code-point
-UChar32 CSSTokenizer::consumeEscape()
+char32_t CSSTokenizer::consumeEscape()
 {
-    UChar cc = consume();
-    ASSERT(!isNewLine(cc));
+    char16_t cc = consume();
+    ASSERT(!isNewline(cc));
     if (isASCIIHexDigit(cc)) {
         unsigned consumedHexDigits = 1;
         StringBuilder hexChars;
         hexChars.append(cc);
-        while (consumedHexDigits < 6 && isASCIIHexDigit(m_input.peekWithoutReplacement(0))) {
+        while (consumedHexDigits < 6 && isASCIIHexDigit(m_input.peek(0))) {
             cc = consume();
             hexChars.append(cc);
             consumedHexDigits++;
         };
         consumeSingleWhitespaceIfNext();
-        bool ok = false;
-        UChar32 codePoint = hexChars.toString().toUIntStrict(&ok, 16);
-        ASSERT(ok);
-        if (!codePoint || (0xD800 <= codePoint && codePoint <= 0xDFFF) || codePoint > 0x10FFFF)
+        auto codePoint = parseInteger<uint32_t>(hexChars, 16).value();
+        if (!codePoint || U_IS_SURROGATE(codePoint) || codePoint > UCHAR_MAX_VALUE)
             return replacementCharacter;
         return codePoint;
     }
@@ -824,17 +828,17 @@ UChar32 CSSTokenizer::consumeEscape()
 
 bool CSSTokenizer::nextTwoCharsAreValidEscape()
 {
-    return twoCharsAreValidEscape(m_input.peekWithoutReplacement(0), m_input.peekWithoutReplacement(1));
+    return twoCharsAreValidEscape(m_input.peek(0), m_input.peek(1));
 }
 
 // http://www.w3.org/TR/css3-syntax/#starts-with-a-number
-bool CSSTokenizer::nextCharsAreNumber(UChar first)
+bool CSSTokenizer::nextCharsAreNumber(char16_t first)
 {
-    UChar second = m_input.peekWithoutReplacement(0);
+    char16_t second = m_input.peek(0);
     if (isASCIIDigit(first))
         return true;
     if (first == '+' || first == '-')
-        return ((isASCIIDigit(second)) || (second == '.' && isASCIIDigit(m_input.peekWithoutReplacement(1))));
+        return ((isASCIIDigit(second)) || (second == '.' && isASCIIDigit(m_input.peek(1))));
     if (first =='.')
         return (isASCIIDigit(second));
     return false;
@@ -842,16 +846,16 @@ bool CSSTokenizer::nextCharsAreNumber(UChar first)
 
 bool CSSTokenizer::nextCharsAreNumber()
 {
-    UChar first = consume();
+    char16_t first = consume();
     bool areNumber = nextCharsAreNumber(first);
     reconsume(first);
     return areNumber;
 }
 
 // http://dev.w3.org/csswg/css-syntax/#would-start-an-identifier
-bool CSSTokenizer::nextCharsAreIdentifier(UChar first)
+bool CSSTokenizer::nextCharsAreIdentifier(char16_t first)
 {
-    UChar second = m_input.peekWithoutReplacement(0);
+    char16_t second = m_input.peek(0);
     if (isNameStartCodePoint(first) || twoCharsAreValidEscape(first, second))
         return true;
 
@@ -863,7 +867,7 @@ bool CSSTokenizer::nextCharsAreIdentifier(UChar first)
 
 bool CSSTokenizer::nextCharsAreIdentifier()
 {
-    UChar first = consume();
+    char16_t first = consume();
     bool areIdentifier = nextCharsAreIdentifier(first);
     reconsume(first);
     return areIdentifier;
