@@ -53,34 +53,32 @@ struct ValueLiteral {
 
     // Synthesize all comparison and equality operators.
 
-    auto operator<=>(const ValueLiteral&) const = default;
+    constexpr auto operator<=>(const ValueLiteral&) const = default;
 
-    // Support binary arithmetic between `ValueLiteral` and machine numeric types.
+    // Support unary operators.
+
+    constexpr ValueLiteral operator+()
+    {
+        return ValueLiteral { value };
+    }
+
+    constexpr ValueLiteral operator-()
+    {
+        return ValueLiteral { -value };
+    }
+
+    // Support addition between `ValueLiteral` and machine numeric types.
 
     constexpr ValueLiteral& operator+=(const ValueLiteral& rhs)
     {
         value += rhs.value;
         return *this;
     }
-
     constexpr ValueLiteral& operator+=(std::convertible_to<double> auto const& rhs)
     {
         value += static_cast<double>(rhs);
         return *this;
     }
-
-    constexpr ValueLiteral& operator-=(const ValueLiteral& rhs)
-    {
-        value -= rhs.value;
-        return *this;
-    }
-
-    constexpr ValueLiteral& operator-=(std::convertible_to<double> auto const& rhs)
-    {
-        value -= static_cast<double>(rhs);
-        return *this;
-    }
-
     friend constexpr ValueLiteral operator+(const ValueLiteral& lhs, const ValueLiteral& rhs)
     {
         return ValueLiteral { lhs.value + rhs.value };
@@ -94,6 +92,18 @@ struct ValueLiteral {
         return ValueLiteral { static_cast<double>(lhs) + rhs.value };
     }
 
+    // Support subtraction between `ValueLiteral` and machine numeric types.
+
+    constexpr ValueLiteral& operator-=(const ValueLiteral& rhs)
+    {
+        value -= rhs.value;
+        return *this;
+    }
+    constexpr ValueLiteral& operator-=(std::convertible_to<double> auto const& rhs)
+    {
+        value -= static_cast<double>(rhs);
+        return *this;
+    }
     friend constexpr ValueLiteral operator-(const ValueLiteral& lhs, const ValueLiteral& rhs)
     {
         return ValueLiteral { lhs.value - rhs.value };
@@ -106,6 +116,34 @@ struct ValueLiteral {
     {
         return ValueLiteral { static_cast<double>(lhs) - rhs.value };
     }
+
+    // Support multiplication between `ValueLiteral` and machine numeric types.
+
+    constexpr ValueLiteral& operator*=(std::convertible_to<double> auto const& rhs)
+    {
+        value *= static_cast<double>(rhs);
+        return *this;
+    }
+    friend constexpr ValueLiteral operator*(const ValueLiteral& lhs, std::convertible_to<double> auto const& rhs)
+    {
+        return ValueLiteral { lhs.value * static_cast<double>(rhs) };
+    }
+    friend constexpr ValueLiteral operator*(std::convertible_to<double> auto const& lhs, const ValueLiteral& rhs)
+    {
+        return ValueLiteral { static_cast<double>(lhs) * rhs.value };
+    }
+
+    // Support division between `ValueLiteral` and machine numeric types.
+
+    constexpr ValueLiteral& operator/=(std::convertible_to<double> auto const& rhs)
+    {
+        value /= static_cast<double>(rhs);
+        return *this;
+    }
+    friend constexpr ValueLiteral operator/(const ValueLiteral& lhs, std::convertible_to<double> auto const& rhs)
+    {
+        return ValueLiteral { lhs.value / static_cast<double>(rhs) };
+    }
 };
 
 #define CSS_DEFINE_UNIT_LITERAL(type, name) \
