@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2011 Google Inc. All Rights Reserved.
- * Copyright (C) 2020 Apple Inc.  All rights reserved.
+ * Copyright (C) 2011 Google Inc. All rights reserved.
+ * Copyright (C) 2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,8 +27,10 @@
 #pragma once
 
 #include "AnimationFrameRate.h"
+#include "Document.h"
 #include "ReducedResolutionSeconds.h"
 #include "Timer.h"
+#include <wtf/CheckedPtr.h>
 #include <wtf/OptionSet.h>
 #include <wtf/RefCounted.h>
 #include <wtf/RefPtr.h>
@@ -36,9 +38,11 @@
 
 namespace WebCore {
 
-class Document;
+class ImminentlyScheduledWorkScope;
 class Page;
 class RequestAnimationFrameCallback;
+class UserGestureToken;
+class WeakPtrImplWithEventTargetData;
 
 class ScriptedAnimationController : public RefCounted<ScriptedAnimationController>
 {
@@ -49,7 +53,6 @@ public:
     }
     ~ScriptedAnimationController();
     void clearDocumentPointer() { m_document = nullptr; }
-    bool requestAnimationFrameEnabled() const;
 
     WEBCORE_EXPORT Seconds interval() const;
     WEBCORE_EXPORT OptionSet<ThrottlingReason> throttlingReasons() const;
@@ -73,11 +76,16 @@ private:
     bool isThrottledRelativeToPage() const;
     bool shouldRescheduleRequestAnimationFrame(ReducedResolutionSeconds) const;
     void scheduleAnimation();
+    RefPtr<Document> protectedDocument();
 
-    using CallbackList = Vector<RefPtr<RequestAnimationFrameCallback>>;
-    CallbackList m_callbacks;
+    struct CallbackData {
+        Ref<RequestAnimationFrameCallback> callback;
+        RefPtr<UserGestureToken> userGestureTokenToForward;
+        RefPtr<ImminentlyScheduledWorkScope> scheduledWorkScope;
+    };
+    Vector<CallbackData> m_callbackDataList;
 
-    WeakPtr<Document> m_document;
+    WeakPtr<Document, WeakPtrImplWithEventTargetData> m_document;
     CallbackId m_nextCallbackId { 0 };
     int m_suspendCount { 0 };
 
