@@ -30,10 +30,11 @@
 
 #include "DocumentFragment.h"
 #include "FrameDestructionObserverInlines.h"
-#include "LocalFrame.h"
+#include "LocalFrameInlines.h"
 #include "NotImplemented.h"
 #include "Pasteboard.h"
 #include "Settings.h"
+#include "SimpleRange.h"
 #include "markup.h"
 
 namespace WebCore {
@@ -46,8 +47,8 @@ static RefPtr<DocumentFragment> createFragmentFromPasteboardData(Pasteboard& pas
     if (types.isEmpty())
         return nullptr;
 
-    if (types.contains("text/html;charset=utf-8"_s) && frame.document()) {
-        String markup = pasteboard.readString("text/html;charset=utf-8"_s);
+    if (types.contains("text/html"_s) && frame.document()) {
+        String markup = pasteboard.readString("text/html"_s);
         return createFragmentFromMarkup(*frame.document(), markup, emptyString(), { });
     }
 
