@@ -27,9 +27,8 @@
 #include "Attribute.h"
 #include "CSSValueKeywords.h"
 #include "CachedImage.h"
-#include "DOMFormData.h"
-#include "ElementIterator.h"
-#include "Frame.h"
+#include "ContainerNodeInlines.h"
+#include "ElementChildIteratorInlines.h"
 #include "FrameLoader.h"
 #include "HTMLDocument.h"
 #include "HTMLFormElement.h"
@@ -37,9 +36,11 @@
 #include "HTMLMetaElement.h"
 #include "HTMLNames.h"
 #include "HTMLParamElement.h"
-#include "HTMLParserIdioms.h"
+#include "LocalFrame.h"
 #include "MIMETypeRegistry.h"
 #include "NodeList.h"
+#include "NodeInlines.h"
+#include "NodeName.h"
 #include "Page.h"
 #include "RenderEmbeddedObject.h"
 #include "RenderImage.h"
@@ -48,23 +49,23 @@
 #include "SubframeLoader.h"
 #include "Text.h"
 #include "Widget.h"
-#include <wtf/IsoMallocInlines.h>
 #include <wtf/Ref.h>
 #include <wtf/RobinHoodHashSet.h>
+#include <wtf/TZoneMallocInlines.h>
 
 #if PLATFORM(IOS_FAMILY)
-#include "RuntimeApplicationChecks.h"
+#include <wtf/RuntimeApplicationChecks.h>
 #endif
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLObjectElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(HTMLObjectElement);
 
 using namespace HTMLNames;
 
 inline HTMLObjectElement::HTMLObjectElement(const QualifiedName& tagName, Document& document, HTMLFormElement* form)
     : HTMLPlugInImageElement(tagName, document)
-    , FormAssociatedElement(form)
+    , FormListedElement(form)
 {
     ASSERT(hasTagName(objectTag));
 }
@@ -99,27 +100,34 @@ void HTMLObjectElement::collectPresentationalHintsForAttribute(const QualifiedNa
         HTMLPlugInImageElement::collectPresentationalHintsForAttribute(name, value, style);
 }
 
-void HTMLObjectElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLObjectElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
+    HTMLPlugInImageElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+
     bool invalidateRenderer = false;
     bool needsWidgetUpdate = false;
 
-    if (name == formAttr)
-        formAttributeChanged();
-    else if (name == typeAttr) {
-        m_serviceType = value.string().left(value.find(';')).convertToASCIILowercase();
+    switch (name.nodeName()) {
+    case AttributeNames::typeAttr:
+        m_serviceType = newValue.string().left(newValue.find(';')).convertToASCIILowercase();
         invalidateRenderer = !hasAttributeWithoutSynchronization(classidAttr);
         needsWidgetUpdate = true;
-    } else if (name == dataAttr) {
-        m_url = stripLeadingAndTrailingHTMLSpaces(value);
+        break;
+    case AttributeNames::dataAttr:
+        // FIXME: trimming whitespace is probably redundant with the URL parser
+        m_url = newValue.string().trim(isASCIIWhitespace);
         invalidateRenderer = !hasAttributeWithoutSynchronization(classidAttr);
         needsWidgetUpdate = true;
         updateImageLoaderWithNewURLSoon();
-    } else if (name == classidAttr) {
+        break;
+    case AttributeNames::classidAttr:
         invalidateRenderer = true;
         needsWidgetUpdate = true;
-    } else
-        HTMLPlugInImageElement::parseAttribute(name, value);
+        break;
+    default:
+        FormListedElement::parseAttribute(name, newValue);
+        break;
+    }
 
     if (needsWidgetUpdate) {
         setNeedsWidgetUpdate(true);
@@ -150,77 +158,24 @@ static void mapDataParamToSrc(Vector<AtomString>& paramNames, Vector<AtomString>
     }
 }
 
-// FIXME: This function should not deal with url or serviceType!
-void HTMLObjectElement::parametersForPlugin(Vector<AtomString>& paramNames, Vector<AtomString>& paramValues, String& url, String& serviceType)
+void HTMLObjectElement::parametersForPlugin(Vector<AtomString>& paramNames, Vector<AtomString>& paramValues)
 {
-    HashSet<StringImpl*, ASCIICaseInsensitiveHash> uniqueParamNames;
-    String urlParameter;
-    
-    // Scan the PARAM children and store their name/value pairs.
-    // Get the URL and type from the params if we don't already have them.
-    for (auto& param : childrenOfType<HTMLParamElement>(*this)) {
-        String name = param.name();
-        if (name.isEmpty())
-            continue;
-
-        uniqueParamNames.add(name.impl());
-        paramNames.append(param.name());
-        paramValues.append(param.value());
-
-        // FIXME: url adjustment does not belong in this function.
-        if (url.isEmpty() && urlParameter.isEmpty() && (equalLettersIgnoringASCIICase(name, "src"_s) || equalLettersIgnoringASCIICase(name, "movie"_s) || equalLettersIgnoringASCIICase(name, "code"_s) || equalLettersIgnoringASCIICase(name, "url"_s)))
-            urlParameter = stripLeadingAndTrailingHTMLSpaces(param.value());
-        // FIXME: serviceType calculation does not belong in this function.
-        if (serviceType.isEmpty() && equalLettersIgnoringASCIICase(name, "type"_s)) {
-            serviceType = param.value();
-            size_t pos = serviceType.find(';');
-            if (pos != notFound)
-                serviceType = serviceType.left(pos);
-        }
-    }
-    
-    // When OBJECT is used for an applet via Sun's Java plugin, the CODEBASE attribute in the tag
-    // points to the Java plugin itself (an ActiveX component) while the actual applet CODEBASE is
-    // in a PARAM tag. See <http://java.sun.com/products/plugin/1.2/docs/tags.html>. This means
-    // we have to explicitly suppress the tag's CODEBASE attribute if there is none in a PARAM,
-    // else our Java plugin will misinterpret it. [4004531]
-    String codebase;
-    if (MIMETypeRegistry::isJavaAppletMIMEType(serviceType)) {
-        codebase = "codebase"_s;
-        uniqueParamNames.add(codebase.impl()); // pretend we found it in a PARAM already
-    }
-    
-    // Turn the attributes of the <object> element into arrays, but don't override <param> values.
     if (hasAttributes()) {
-        for (const Attribute& attribute : attributesIterator()) {
-            const AtomString& name = attribute.name().localName();
-            if (!uniqueParamNames.contains(name.impl())) {
-                paramNames.append(name);
-                paramValues.append(attribute.value());
-            }
+        for (auto& attribute : attributes()) {
+            paramNames.append(attribute.name().localName());
+            paramValues.append(attribute.value());
         }
     }
-    
-    mapDataParamToSrc(paramNames, paramValues);
-    
-    // HTML5 says that an object resource's URL is specified by the object's data
-    // attribute, not by a param element. However, for compatibility, allow the
-    // resource's URL to be given by a param named "src", "movie", "code" or "url"
-    // if we know that resource points to a plug-in.
 
-    if (url.isEmpty() && !urlParameter.isEmpty()) {
-        auto& loader = document().frame()->loader().subframeLoader();
-        if (loader.resourceWillUsePlugin(urlParameter, serviceType))
-            url = urlParameter;
-    }
+    mapDataParamToSrc(paramNames, paramValues);
 }
 
 bool HTMLObjectElement::hasFallbackContent() const
 {
     for (RefPtr<Node> child = firstChild(); child; child = child->nextSibling()) {
         // Ignore whitespace-only text, and <param> tags, any other content is fallback content.
-        if (is<Text>(*child)) {
-            if (!downcast<Text>(*child).data().isAllSpecialCharacters<isHTMLSpace>())
+        if (auto* textChild = dynamicDowncast<Text>(*child)) {
+            if (!textChild->containsOnlyASCIIWhitespace())
                 return true;
         } else if (!is<HTMLParamElement>(*child))
             return true;
@@ -228,16 +183,6 @@ bool HTMLObjectElement::hasFallbackContent() const
     return false;
 }
 
-bool HTMLObjectElement::hasValidClassId()
-{
-    if (MIMETypeRegistry::isJavaAppletMIMEType(serviceType()) && protocolIs(attributeWithoutSynchronization(classidAttr), "java"_s))
-        return true;
-
-    // HTML5 says that fallback content should be rendered if a non-empty
-    // classid is specified for which the UA can't find a suitable plug-in.
-    return attributeWithoutSynchronization(classidAttr).isEmpty();
-}
-
 // FIXME: This should be unified with HTMLEmbedElement::updateWidget and
 // moved down into HTMLPluginImageElement.cpp
 void HTMLObjectElement::updateWidget(CreatePlugins createPlugins)
@@ -259,15 +204,12 @@ void HTMLObjectElement::updateWidget(CreatePlugins createPlugins)
         return;
     }
 
-    String url = this->url();
-    String serviceType = this->serviceType();
-
     // FIXME: These should be joined into a PluginParameters class.
     Vector<AtomString> paramNames;
     Vector<AtomString> paramValues;
-    parametersForPlugin(paramNames, paramValues, url, serviceType);
+    parametersForPlugin(paramNames, paramValues);
 
-    // Note: url is modified above by parametersForPlugin.
+    auto url = this->url();
     if (!canLoadURL(url)) {
         setNeedsWidgetUpdate(false);
         return;
@@ -275,6 +217,7 @@ void HTMLObjectElement::updateWidget(CreatePlugins createPlugins)
 
     // FIXME: It's unfortunate that we have this special case here.
     // See http://trac.webkit.org/changeset/25128 and the plugins/netscape-plugin-setwindow-size.html test.
+    auto serviceType = this->serviceType();
     if (createPlugins == CreatePlugins::No && wouldLoadAsPlugIn(url, serviceType))
         return;
 
@@ -282,9 +225,12 @@ void HTMLObjectElement::updateWidget(CreatePlugins createPlugins)
 
     Ref protectedThis = *this; // Plugin loading can make arbitrary DOM mutations.
 
+    // HTML5 says that fallback content should be rendered if a non-empty
+    // classid is specified for which the UA can't find a suitable plug-in.
+    //
     // Dispatching a beforeLoad event could have executed code that changed the document.
     // Make sure the URL is still safe to load.
-    bool success = hasValidClassId() && canLoadURL(url);
+    bool success = attributeWithoutSynchronization(classidAttr).isEmpty() && canLoadURL(url);
     if (success)
         success = requestObject(url, serviceType, paramNames, paramValues);
     if (!success && hasFallbackContent())
@@ -294,7 +240,9 @@ void HTMLObjectElement::updateWidget(CreatePlugins createPlugins)
 Node::InsertedIntoAncestorResult HTMLObjectElement::insertedIntoAncestor(InsertionType insertionType, ContainerNode& parentOfInsertedTree)
 {
     HTMLPlugInImageElement::insertedIntoAncestor(insertionType, parentOfInsertedTree);
-    FormAssociatedElement::insertedIntoAncestor(insertionType, parentOfInsertedTree);
+    FormListedElement::elementInsertedIntoAncestor(*this, insertionType);
+    if (!insertionType.connectedToDocument)
+        return InsertedIntoAncestorResult::Done;
     return InsertedIntoAncestorResult::NeedsPostInsertionCallback;
 }
 
@@ -306,7 +254,7 @@ void HTMLObjectElement::didFinishInsertingNode()
 void HTMLObjectElement::removedFromAncestor(RemovalType removalType, ContainerNode& oldParentOfRemovedTree)
 {
     HTMLPlugInImageElement::removedFromAncestor(removalType, oldParentOfRemovedTree);
-    FormAssociatedElement::removedFromAncestor(removalType, oldParentOfRemovedTree);
+    FormListedElement::elementRemovedFromAncestor(*this, removalType);
 }
 
 void HTMLObjectElement::childrenChanged(const ChildChange& change)
@@ -334,15 +282,20 @@ void HTMLObjectElement::renderFallbackContent()
 {
     if (m_useFallbackContent)
         return;
-    
+
     if (!isConnected())
         return;
 
     scheduleUpdateForAfterStyleResolution();
     invalidateStyleAndRenderersForSubtree();
 
+    // Presence of a UA shadow root indicates render invalidation during embedded PDF plugin bringup, and not a failed render.
+    // It's safe to special case here because UA shadow root cannot be attached to <object>/<embed> programmatically.
+    if (userAgentShadowRoot())
+        return;
+
     // Before we give up and use fallback content, check to see if this is a MIME type issue.
-    auto* loader = imageLoader();
+    RefPtr loader = imageLoader();
     if (loader && loader->image() && loader->image()->status() != CachedResource::LoadError) {
         m_serviceType = loader->image()->response().mimeType();
         if (!isImageType()) {
@@ -359,23 +312,23 @@ static inline bool preventsParentObjectFromExposure(const Element& child)
 {
     static NeverDestroyed mostKnownTags = [] {
         MemoryCompactLookupOnlyRobinHoodHashSet<QualifiedName> set;
-        auto* tags = HTMLNames::getHTMLTags();
-        set.reserveInitialCapacity(HTMLNames::HTMLTagsCount);
-        for (size_t i = 0; i < HTMLNames::HTMLTagsCount; i++) {
-            auto& tag = *tags[i];
+        auto tags = HTMLNames::getHTMLTags();
+        set.reserveInitialCapacity(tags.size());
+        for (auto* tagPtr : tags) {
+            auto& tag = *tagPtr;
             // Only the param element was explicitly mentioned in the HTML specification rule
             // we were trying to implement, but these are other known HTML elements that we
             // have decided, over the years, to treat as children that do not prevent object
             // names from being exposed.
             if (tag == bgsoundTag
-                || tag == commandTag
                 || tag == detailsTag
                 || tag == figcaptionTag
                 || tag == figureTag
                 || tag == paramTag
                 || tag == summaryTag
-                || tag == trackTag)
+                || tag == trackTag) {
                 continue;
+            }
             set.add(tag);
         }
         return set;
@@ -385,10 +338,10 @@ static inline bool preventsParentObjectFromExposure(const Element& child)
 
 static inline bool preventsParentObjectFromExposure(const Node& child)
 {
-    if (is<Element>(child))
-        return preventsParentObjectFromExposure(downcast<Element>(child));
-    if (is<Text>(child))
-        return !downcast<Text>(child).data().isAllSpecialCharacters<isHTMLSpace>();
+    if (auto* childElement = dynamicDowncast<Element>(child))
+        return preventsParentObjectFromExposure(*childElement);
+    if (auto* childText = dynamicDowncast<Text>(child))
+        return !childText->containsOnlyASCIIWhitespace();
     return true;
 }
 
@@ -412,45 +365,27 @@ void HTMLObjectElement::updateExposedState()
 {
     bool wasExposed = std::exchange(m_isExposed, shouldBeExposed(*this));
 
-    if (m_isExposed != wasExposed && isConnected() && !isInShadowTree() && is<HTMLDocument>(document())) {
-        auto& document = downcast<HTMLDocument>(this->document());
+    if (m_isExposed != wasExposed && isConnected() && !isInShadowTree()) {
+        if (RefPtr document = dynamicDowncast<HTMLDocument>(this->document())) {
+            auto& id = getIdAttribute();
+            if (!id.isEmpty()) {
+                if (m_isExposed)
+                    document->addDocumentNamedItem(id, *this);
+                else
+                    document->removeDocumentNamedItem(id, *this);
+            }
 
-        auto& id = getIdAttribute();
-        if (!id.isEmpty()) {
-            if (m_isExposed)
-                document.addDocumentNamedItem(*id.impl(), *this);
-            else
-                document.removeDocumentNamedItem(*id.impl(), *this);
-        }
-
-        auto& name = getNameAttribute();
-        if (!name.isEmpty() && id != name) {
-            if (m_isExposed)
-                document.addDocumentNamedItem(*name.impl(), *this);
-            else
-                document.removeDocumentNamedItem(*name.impl(), *this);
+            auto& name = getNameAttribute();
+            if (!name.isEmpty() && id != name) {
+                if (m_isExposed)
+                    document->addDocumentNamedItem(name, *this);
+                else
+                    document->removeDocumentNamedItem(name, *this);
+            }
         }
     }
 }
 
-bool HTMLObjectElement::containsJavaApplet() const
-{
-    if (MIMETypeRegistry::isJavaAppletMIMEType(attributeWithoutSynchronization(typeAttr)))
-        return true;
-
-    for (auto& child : childrenOfType<Element>(*this)) {
-        if (child.hasTagName(paramTag) && equalLettersIgnoringASCIICase(child.getNameAttribute(), "type"_s)
-            && MIMETypeRegistry::isJavaAppletMIMEType(child.attributeWithoutSynchronization(valueAttr).string()))
-            return true;
-        if (child.hasTagName(objectTag) && downcast<HTMLObjectElement>(child).containsJavaApplet())
-            return true;
-        if (child.hasTagName(appletTag))
-            return true;
-    }
-    
-    return false;
-}
-
 void HTMLObjectElement::addSubresourceAttributeURLs(ListHashSet<URL>& urls) const
 {
     HTMLPlugInImageElement::addSubresourceAttributeURLs(urls);
@@ -460,7 +395,7 @@ void HTMLObjectElement::addSubresourceAttributeURLs(ListHashSet<URL>& urls) cons
 
 void HTMLObjectElement::didMoveToNewDocument(Document& oldDocument, Document& newDocument)
 {
-    FormAssociatedElement::didMoveToNewDocument(oldDocument);
+    FormListedElement::didMoveToNewDocument();
     HTMLPlugInImageElement::didMoveToNewDocument(oldDocument, newDocument);
 }
 
