@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2025 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -57,8 +57,8 @@ private:
     
     void drawCrossfade(GraphicsContext&);
 
-    Ref<Image> m_fromImage;
-    Ref<Image> m_toImage;
+    const Ref<Image> m_fromImage;
+    const Ref<Image> m_toImage;
 
     float m_percentage;
     FloatSize m_crossfadeSize;
