@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014 Apple Inc.  All rights reserved.
+ * Copyright (C) 2014 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -20,17 +20,27 @@
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
- * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "config.h"
 #include "PlatformTimeRanges.h"
 
 #include <math.h>
+#include <numeric>
+#include <wtf/NeverDestroyed.h>
 #include <wtf/PrintStream.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
-    
+
+WTF_MAKE_TZONE_ALLOCATED_IMPL(PlatformTimeRanges);
+
+PlatformTimeRanges::PlatformTimeRanges()
+{
+}
+
 PlatformTimeRanges::PlatformTimeRanges(const MediaTime& start, const MediaTime& end)
 {
     add(start, end);
@@ -41,6 +51,21 @@ PlatformTimeRanges::PlatformTimeRanges(Vector<Range>&& ranges)
 {
 }
 
+const PlatformTimeRanges& PlatformTimeRanges::emptyRanges()
+{
+    static LazyNeverDestroyed<PlatformTimeRanges> emptyRanges;
+    static std::once_flag onceKey;
+    std::call_once(onceKey, [&] {
+        emptyRanges.construct();
+    });
+    return emptyRanges.get();
+}
+
+MediaTime PlatformTimeRanges::timeFudgeFactor()
+{
+    return { 2002, 24000 };
+}
+
 void PlatformTimeRanges::invert()
 {
     PlatformTimeRanges inverted;
@@ -50,14 +75,14 @@ void PlatformTimeRanges::invert()
     if (!m_ranges.size())
         inverted.add(negInf, posInf);
     else {
-        MediaTime start = m_ranges.first().m_start;
+        MediaTime start = m_ranges.first().start;
         if (start != negInf)
             inverted.add(negInf, start);
 
         for (size_t index = 0; index + 1 < m_ranges.size(); ++index)
-            inverted.add(m_ranges[index].m_end, m_ranges[index + 1].m_start);
+            inverted.add(m_ranges[index].end, m_ranges[index + 1].start);
 
-        MediaTime end = m_ranges.last().m_end;
+        MediaTime end = m_ranges.last().end;
         if (end != posInf)
             inverted.add(end, posInf);
     }
@@ -81,12 +106,43 @@ void PlatformTimeRanges::unionWith(const PlatformTimeRanges& other)
 
     for (size_t index = 0; index < other.m_ranges.size(); ++index) {
         const Range& range = other.m_ranges[index];
-        unioned.add(range.m_start, range.m_end);
+        unioned.add(range.start, range.end);
     }
 
     m_ranges.swap(unioned.m_ranges);
 }
 
+PlatformTimeRanges& PlatformTimeRanges::operator+=(const PlatformTimeRanges& other)
+{
+    unionWith(other);
+    return *this;
+}
+
+PlatformTimeRanges& PlatformTimeRanges::operator-=(const PlatformTimeRanges& other)
+{
+    for (const auto& range : other.m_ranges)
+        *this -= range;
+
+    return *this;
+}
+
+PlatformTimeRanges& PlatformTimeRanges::operator-=(const Range& range)
+{
+    if (range.isEmpty() || m_ranges.isEmpty())
+        return *this;
+
+    auto firstEnd = std::max(m_ranges[0].start, range.start);
+    auto secondStart = std::min(m_ranges.last().end, range.end);
+    Vector<Range> ranges { 2 };
+    if (m_ranges[0].start != firstEnd)
+        ranges.append({ m_ranges[0].start, firstEnd });
+    if (secondStart != m_ranges.last().end)
+        ranges.append({ secondStart, m_ranges.last().end });
+    intersectWith(WTFMove(ranges));
+
+    return *this;
+}
+
 MediaTime PlatformTimeRanges::start(unsigned index) const
 {
     bool ignoredValid;
@@ -94,14 +150,14 @@ MediaTime PlatformTimeRanges::start(unsigned index) const
 }
 
 MediaTime PlatformTimeRanges::start(unsigned index, bool& valid) const
-{ 
+{
     if (index >= length()) {
         valid = false;
         return MediaTime::zeroTime();
     }
-    
+
     valid = true;
-    return m_ranges[index].m_start;
+    return m_ranges[index].start;
 }
 
 MediaTime PlatformTimeRanges::end(unsigned index) const
@@ -111,14 +167,14 @@ MediaTime PlatformTimeRanges::end(unsigned index) const
 }
 
 MediaTime PlatformTimeRanges::end(unsigned index, bool& valid) const
-{ 
+{
     if (index >= length()) {
         valid = false;
         return MediaTime::zeroTime();
     }
 
     valid = true;
-    return m_ranges[index].m_end;
+    return m_ranges[index].end;
 }
 
 MediaTime PlatformTimeRanges::duration(unsigned index) const
@@ -126,7 +182,7 @@ MediaTime PlatformTimeRanges::duration(unsigned index) const
     if (index >= length())
         return MediaTime::invalidTime();
 
-    return m_ranges[index].m_end - m_ranges[index].m_start;
+    return m_ranges[index].end - m_ranges[index].start;
 }
 
 MediaTime PlatformTimeRanges::maximumBufferedTime() const
@@ -134,7 +190,7 @@ MediaTime PlatformTimeRanges::maximumBufferedTime() const
     if (!length())
         return MediaTime::invalidTime();
 
-    return m_ranges[length() - 1].m_end;
+    return m_ranges[length() - 1].end;
 }
 
 MediaTime PlatformTimeRanges::minimumBufferedTime() const
@@ -142,32 +198,41 @@ MediaTime PlatformTimeRanges::minimumBufferedTime() const
     if (!length())
         return MediaTime::invalidTime();
 
-    return m_ranges[0].m_start;
+    return m_ranges[0].start;
 }
 
-void PlatformTimeRanges::add(const MediaTime& start, const MediaTime& end)
+void PlatformTimeRanges::add(const MediaTime& start, const MediaTime& end, AddTimeRangeOption addTimeRangeOption)
 {
-#if !PLATFORM(MAC) // https://bugs.webkit.org/show_bug.cgi?id=180253
     ASSERT(start.isValid());
     ASSERT(end.isValid());
-#endif
     ASSERT(start <= end);
 
-    unsigned overlappingArcIndex;
-    Range addedRange(start, end);
+    auto startTime = start;
+    auto endTime = end;
+    if (addTimeRangeOption == AddTimeRangeOption::EliminateSmallGaps) {
+        // Eliminate small gaps between buffered ranges by coalescing
+        // disjoint ranges separated by less than a "fudge factor".
+        auto nearestToPresentationStartTime = nearest(startTime);
+        if (nearestToPresentationStartTime.isValid() && (startTime - nearestToPresentationStartTime).isBetween(MediaTime::zeroTime(), timeFudgeFactor()))
+            startTime = nearestToPresentationStartTime;
+
+        auto nearestToPresentationEndTime = nearest(endTime);
+        if (nearestToPresentationEndTime.isValid() && (nearestToPresentationEndTime - endTime).isBetween(MediaTime::zeroTime(), timeFudgeFactor()))
+            endTime = nearestToPresentationEndTime;
+    }
+    size_t overlappingArcIndex;
+    Range addedRange { .start = startTime, .end = endTime };
 
     // For each present range check if we need to:
     // - merge with the added range, in case we are overlapping or contiguous
     // - Need to insert in place, we we are completely, not overlapping and not contiguous
     // in between two ranges.
-    //
-    // TODO: Given that we assume that ranges are correctly ordered, this could be optimized.
 
-    for (overlappingArcIndex = 0; overlappingArcIndex < m_ranges.size(); overlappingArcIndex++) {
+    for (overlappingArcIndex = findLastRangeIndexBefore(start, end); overlappingArcIndex < m_ranges.size(); overlappingArcIndex++) {
         if (addedRange.isOverlappingRange(m_ranges[overlappingArcIndex]) || addedRange.isContiguousWithRange(m_ranges[overlappingArcIndex])) {
             // We need to merge the addedRange and that range.
             addedRange = addedRange.unionWithOverlappingOrContiguousRange(m_ranges[overlappingArcIndex]);
-            m_ranges.remove(overlappingArcIndex);
+            m_ranges.removeAt(overlappingArcIndex);
             overlappingArcIndex--;
         } else {
             // Check the case for which there is no more to do
@@ -228,13 +293,13 @@ PlatformTimeRanges PlatformTimeRanges::copyWithEpsilon(const MediaTime& epsilon)
     Vector<Range> ranges;
     unsigned n1 = 0;
     for (unsigned n2 = 1; n2 < length(); n2++) {
-        auto& previousRangeEnd = m_ranges[n2 - 1].m_end;
-        if (previousRangeEnd + epsilon < m_ranges[n2].m_start) {
-            ranges.append({ m_ranges[n1].m_start, previousRangeEnd });
+        auto& previousRangeEnd = m_ranges[n2 - 1].end;
+        if (previousRangeEnd + epsilon < m_ranges[n2].start) {
+            ranges.append({ m_ranges[n1].start, previousRangeEnd });
             n1 = n2;
         }
     }
-    ranges.append({ m_ranges[n1].m_start, m_ranges[length() - 1].m_end });
+    ranges.append({ m_ranges[n1].start, m_ranges[length() - 1].end });
     return ranges;
 }
 
@@ -287,4 +352,41 @@ void PlatformTimeRanges::dump(PrintStream& out) const
         out.print("[", start(i), "..", end(i), "] ");
 }
 
+String PlatformTimeRanges::toString() const
+{
+    StringBuilder result;
+
+    for (size_t i = 0; i < length(); ++i)
+        result.append('[', start(i).toString(), ".."_s, end(i).toString(), "] "_s);
+
+    return result.toString();
+}
+
+size_t PlatformTimeRanges::findLastRangeIndexBefore(const MediaTime& start, const MediaTime& end) const
+{
+    ASSERT(start <= end);
+
+    if (m_ranges.isEmpty())
+        return 0;
+
+    const Range range { .start = start, .end = end };
+    size_t first, last, middle;
+    size_t index = 0;
+
+    first = 0;
+    last = m_ranges.size() - 1;
+    middle = std::midpoint(first, last);
+
+    while (first < last && middle > 0) {
+        if (m_ranges[middle].isBeforeRange(range)) {
+            index = middle;
+            first = middle + 1;
+        } else
+            last = middle - 1;
+
+        middle = std::midpoint(first, last);
+    }
+    return index;
+}
+
 }
