@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 Apple Inc.  All rights reserved.
+ * Copyright (C) 2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -27,6 +27,8 @@
 #include "SourceImage.h"
 
 #include "GraphicsContext.h"
+#include "ImageBuffer.h"
+#include "NativeImage.h"
 
 namespace WebCore {
 
@@ -35,6 +37,17 @@ SourceImage::SourceImage(ImageVariant&& imageVariant)
 {
 }
 
+SourceImage::SourceImage(const SourceImage&) = default;
+SourceImage::SourceImage(SourceImage&&) = default;
+SourceImage& SourceImage::operator=(const SourceImage&) = default;
+SourceImage& SourceImage::operator=(SourceImage&&) = default;
+SourceImage::~SourceImage() = default;
+
+bool SourceImage::operator==(const SourceImage& other) const
+{
+    return imageIdentifier() == other.imageIdentifier();
+}
+
 static inline NativeImage* nativeImageOf(const SourceImage::ImageVariant& imageVariant)
 {
     if (auto* nativeImage = std::get_if<Ref<NativeImage>>(&imageVariant))
@@ -55,7 +68,7 @@ NativeImage* SourceImage::nativeImage() const
     if (!m_transformedImageVariant) {
         auto imageBuffer = std::get<Ref<ImageBuffer>>(m_imageVariant);
 
-        auto nativeImage = imageBuffer->copyNativeImage(DontCopyBackingStore);
+        auto nativeImage = imageBuffer->createNativeImageReference();
         if (!nativeImage)
             return nullptr;
 
@@ -87,11 +100,11 @@ ImageBuffer* SourceImage::imageBuffer() const
         auto nativeImage = std::get<Ref<NativeImage>>(m_imageVariant);
 
         auto rect = FloatRect { { }, nativeImage->size() };
-        auto imageBuffer = ImageBuffer::create(nativeImage->size(), RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), PixelFormat::BGRA8);
+        auto imageBuffer = ImageBuffer::create(nativeImage->size(), RenderingMode::Unaccelerated, RenderingPurpose::Unspecified, 1, DestinationColorSpace::SRGB(), ImageBufferPixelFormat::BGRA8);
         if (!imageBuffer)
             return nullptr;
 
-        imageBuffer->context().drawNativeImage(nativeImage, rect.size(), rect, rect);
+        imageBuffer->context().drawNativeImage(nativeImage, rect, rect);
         m_transformedImageVariant = { imageBuffer.releaseNonNull() };
     }
 
