@@ -26,15 +26,14 @@
  */
 
 #include "config.h"
-#include "GraphicsContextGLOpenGL.h"
+#include "GraphicsContextGL.h"
 
 #if ENABLE(WEBGL) && USE(CAIRO)
 
+#include "BitmapImage.h"
 #include "CairoUtilities.h"
 #include "GraphicsContext.h"
 #include "GraphicsContextGLImageExtractor.h"
-#include "Image.h"
-#include "ImageSource.h"
 #include "PixelBuffer.h"
 #include "RefPtrCairo.h"
 #include <cairo.h>
@@ -45,21 +44,19 @@ GraphicsContextGLImageExtractor::~GraphicsContextGLImageExtractor() = default;
 
 bool GraphicsContextGLImageExtractor::extractImage(bool premultiplyAlpha, bool ignoreGammaAndColorProfile, bool)
 {
-    if (!m_image)
-        return false;
     // We need this to stay in scope because the native image is just a shallow copy of the data.
     AlphaOption alphaOption = premultiplyAlpha ? AlphaOption::Premultiplied : AlphaOption::NotPremultiplied;
     GammaAndColorProfileOption gammaAndColorProfileOption = ignoreGammaAndColorProfile ? GammaAndColorProfileOption::Ignored : GammaAndColorProfileOption::Applied;
-    auto source = ImageSource::create(nullptr, alphaOption, gammaAndColorProfileOption);
+    auto image = BitmapImage::create(nullptr, alphaOption, gammaAndColorProfileOption);
     m_alphaOp = AlphaOp::DoNothing;
 
     if (m_image->data()) {
-        source->setData(m_image->data(), true);
-        if (!source->frameCount())
+        image->setData(m_image->data(), true);
+        if (!image->frameCount())
             return false;
-        m_imageSurface = source->createFrameImageAtIndex(0)->platformImage();
+        m_imageSurface = image->currentNativeImage()->platformImage();
     } else {
-        m_imageSurface = m_image->nativeImageForCurrentFrame()->platformImage();
+        m_imageSurface = m_image->currentNativeImage()->platformImage();
         // 1. For texImage2D with HTMLVideoElment input, assume no PremultiplyAlpha had been applied and the alpha value is 0xFF for each pixel,
         // which is true at present and may be changed in the future and needs adjustment accordingly.
         // 2. For texImage2D with HTMLCanvasElement input in which Alpha is already Premultiplied in this port, 
@@ -100,21 +97,20 @@ bool GraphicsContextGLImageExtractor::extractImage(bool premultiplyAlpha, bool i
             ++srcUnpackAlignment;
     }
 
-    m_imagePixelData = cairo_image_surface_get_data(m_imageSurface.get());
+    m_imagePixelData = span(m_imageSurface.get());
     m_imageSourceFormat = DataFormat::BGRA8;
     m_imageSourceUnpackAlignment = srcUnpackAlignment;
     return true;
 }
 
-void GraphicsContextGL::paintToCanvas(const GraphicsContextGLAttributes& sourceContextAttributes, Ref<PixelBuffer>&& pixelBuffer, const IntSize& canvasSize, GraphicsContext& context)
+RefPtr<NativeImage> GraphicsContextGL::createNativeImageFromPixelBuffer(const GraphicsContextGLAttributes& sourceContextAttributes, Ref<PixelBuffer>&& pixelBuffer)
 {
     ASSERT(!pixelBuffer->size().isEmpty());
-    if (canvasSize.isEmpty())
-        return;
 
     // Convert RGBA to BGRA. BGRA is CAIRO_FORMAT_ARGB32 on little-endian architectures.
-    size_t totalBytes = pixelBuffer->sizeInBytes();
-    uint8_t* pixels = pixelBuffer->bytes();
+    Ref protectedPixelBuffer = pixelBuffer;
+    size_t totalBytes = pixelBuffer->bytes().size();
+    uint8_t* pixels = pixelBuffer->bytes().data();
     for (size_t i = 0; i < totalBytes; i += 4)
         std::swap(pixels[i], pixels[i + 2]);
 
@@ -127,17 +123,13 @@ void GraphicsContextGL::paintToCanvas(const GraphicsContextGLAttributes& sourceC
     }
 
     auto imageSize = pixelBuffer->size();
-
     RefPtr<cairo_surface_t> imageSurface = adoptRef(cairo_image_surface_create_for_data(
-        pixelBuffer->bytes(), CAIRO_FORMAT_ARGB32, imageSize.width(), imageSize.height(), imageSize.width() * 4));
-
-    auto image = NativeImage::create(WTFMove(imageSurface));
-
-    GraphicsContextStateSaver stateSaver(context);
-    context.scale(FloatSize(1, -1));
-    context.translate(0, -imageSize.height());
-    context.setImageInterpolationQuality(InterpolationQuality::DoNotInterpolate);
-    context.drawNativeImage(*image, imageSize, FloatRect({ }, canvasSize), FloatRect({ }, imageSize), { CompositeOperator::Copy });
+        pixels, CAIRO_FORMAT_ARGB32, imageSize.width(), imageSize.height(), imageSize.width() * 4));
+    static cairo_user_data_key_t dataKey;
+    cairo_surface_set_user_data(imageSurface.get(), &dataKey, &protectedPixelBuffer.leakRef(), [](void* buffer) {
+        static_cast<PixelBuffer*>(buffer)->deref();
+    });
+    return NativeImage::create(WTFMove(imageSurface));
 }
 
 } // namespace WebCore
