@@ -4,7 +4,7 @@
  * Copyright (C) 2005 Eric Seidel <eric@webkit.org>
  * Copyright (C) 2009 Dirk Schulze <krit@webkit.org>
  * Copyright (C) Research In Motion Limited 2010. All rights reserved.
- * Copyright (C) 2021-2022 Apple Inc.  All rights reserved.
+ * Copyright (C) 2021-2022 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -31,18 +31,25 @@
 
 namespace WebCore {
 
-Ref<FEOffset> FEOffset::create(float dx, float dy)
+Ref<FEOffset> FEOffset::create(float dx, float dy, DestinationColorSpace colorSpace)
 {
-    return adoptRef(*new FEOffset(dx, dy));
+    return adoptRef(*new FEOffset(dx, dy, colorSpace));
 }
 
-FEOffset::FEOffset(float dx, float dy)
-    : FilterEffect(FilterEffect::Type::FEOffset)
+FEOffset::FEOffset(float dx, float dy, DestinationColorSpace colorSpace)
+    : FilterEffect(FilterEffect::Type::FEOffset, colorSpace)
     , m_dx(dx)
     , m_dy(dy)
 {
 }
 
+bool FEOffset::operator==(const FEOffset& other) const
+{
+    return FilterEffect::operator==(other)
+        && m_dx == other.m_dx
+        && m_dy == other.m_dy;
+}
+
 bool FEOffset::setDx(float dx)
 {
     if (m_dx == dx)
@@ -59,9 +66,9 @@ bool FEOffset::setDy(float dy)
     return true;
 }
 
-FloatRect FEOffset::calculateImageRect(const Filter& filter, const FilterImageVector& inputs, const FloatRect& primitiveSubregion) const
+FloatRect FEOffset::calculateImageRect(const Filter& filter, std::span<const FloatRect> inputImageRects, const FloatRect& primitiveSubregion) const
 {
-    auto imageRect = inputs[0]->imageRect();
+    auto imageRect = inputImageRects[0];
     imageRect.move(filter.resolvedSize({ m_dx, m_dy }));
     return filter.clipToMaxEffectRect(imageRect, primitiveSubregion);
 }
@@ -83,7 +90,7 @@ IntOutsets FEOffset::calculateOutsets(const FloatSize& offset)
     return outsets;
 }
 
-bool FEOffset::resultIsAlphaImage(const FilterImageVector& inputs) const
+bool FEOffset::resultIsAlphaImage(std::span<const Ref<FilterImage>> inputs) const
 {
     return inputs[0]->isAlphaImage();
 }
@@ -95,12 +102,12 @@ std::unique_ptr<FilterEffectApplier> FEOffset::createSoftwareApplier() const
 
 TextStream& FEOffset::externalRepresentation(TextStream& ts, FilterRepresentation representation) const
 {
-    ts << indent << "[feOffset";
+    ts << indent << "[feOffset"_s;
     FilterEffect::externalRepresentation(ts, representation);
 
-    ts << " dx=\"" << dx() << "\" dy=\"" << dy() << "\"";
+    ts << " dx=\""_s << dx() << "\" dy=\""_s << dy() << '"';
 
-    ts << "]\n";
+    ts << "]\n"_s;
     return ts;
 }
 
