@@ -19,8 +19,6 @@
 
 #pragma once
 
-#if ENABLE(SVG_FONTS)
-
 #include "SVGElement.h"
 
 namespace WebCore {
@@ -28,11 +26,12 @@ namespace WebCore {
 class CSSValueList;
 
 class SVGFontFaceSrcElement final : public SVGElement {
-    WTF_MAKE_ISO_ALLOCATED(SVGFontFaceSrcElement);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(SVGFontFaceSrcElement);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(SVGFontFaceSrcElement);
 public:
     static Ref<SVGFontFaceSrcElement> create(const QualifiedName&, Document&);
 
-    Ref<CSSValueList> srcValue() const;
+    Ref<CSSValueList> createSrcValue() const;
     
 private:
     SVGFontFaceSrcElement(const QualifiedName&, Document&);
@@ -42,5 +41,3 @@ private:
 };
 
 } // namespace WebCore
-
-#endif // ENABLE(SVG_FONTS)
