@@ -143,7 +143,10 @@ void WebSharedWorkerServerConnection::postErrorToWorkerObject(WebCore::SharedWor
 
 std::optional<SharedPreferencesForWebProcess> WebSharedWorkerServerConnection::sharedPreferencesForWebProcess(const IPC::Connection& connection) const
 {
-    return m_networkProcess->webProcessConnection(connection)->sharedPreferencesForWebProcess();
+    RefPtr webProcessConnection = m_networkProcess->protectedWebProcessConnection(connection);
+    if (!webProcessConnection)
+        return std::nullopt;
+    return webProcessConnection->sharedPreferencesForWebProcess();
 }
 
 #if ENABLE(CONTENT_EXTENSIONS)
