@@ -27,13 +27,17 @@
 #include "WebSocketTaskSoup.h"
 
 #include "NetworkProcess.h"
+#include "NetworkSession.h"
 #include "NetworkSocketChannel.h"
+#include <WebCore/AuthenticationChallenge.h>
 #include <WebCore/HTTPParsers.h>
 #include <WebCore/ResourceRequest.h>
 #include <WebCore/ResourceResponse.h>
 #include <WebCore/SoupVersioning.h>
-#include <WebCore/WebSocketChannel.h>
+#include <WebCore/ThreadableWebSocketChannel.h>
 #include <wtf/RunLoop.h>
+#include <wtf/TZoneMallocInlines.h>
+#include <wtf/glib/GSpanExtras.h>
 #include <wtf/glib/GUniquePtr.h>
 #include <wtf/glib/RunLoopSourcePriority.h>
 #include <wtf/text/StringBuilder.h>
@@ -53,27 +57,46 @@ static inline bool isConnectionError(GError* error, SoupMessage* message)
 #endif
 }
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(WebSocketTask);
+
 WebSocketTask::WebSocketTask(NetworkSocketChannel& channel, const WebCore::ResourceRequest& request, SoupSession* session, SoupMessage* msg, const String& protocol)
     : m_channel(channel)
     , m_request(request)
     , m_handshakeMessage(msg)
     , m_cancellable(adoptGRef(g_cancellable_new()))
-    , m_delayFailTimer(RunLoop::main(), this, &WebSocketTask::delayFailTimerFired)
+    , m_delayFailTimer(RunLoop::mainSingleton(), "WebSocketTask::DelayFailTimer"_s, this, &WebSocketTask::delayFailTimerFired)
 {
+    WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN // GTK/WPE port
     auto protocolList = protocol.split(',');
     GUniquePtr<char*> protocols;
     if (!protocolList.isEmpty()) {
         protocols.reset(static_cast<char**>(g_new0(char*, protocolList.size() + 1)));
         unsigned i = 0;
         for (auto& subprotocol : protocolList)
-            protocols.get()[i++] = g_strdup(WebCore::stripLeadingAndTrailingHTTPSpaces(subprotocol).utf8().data());
+            protocols.get()[i++] = g_strdup(subprotocol.trim(isASCIIWhitespaceWithoutFF<char16_t>).utf8().data());
     }
+    WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
 #if USE(SOUP2)
     // Ensure a new connection is used for WebSockets.
     // FIXME: this is done by libsoup since 2.69.1 and 2.68.4, so it can be removed when bumping the libsoup requirement.
     // See https://bugs.webkit.org/show_bug.cgi?id=203404
     soup_message_set_flags(msg, static_cast<SoupMessageFlags>(soup_message_get_flags(msg) | SOUP_MESSAGE_NEW_CONNECTION));
+#else
+    {
+        // No need to subscribe to the "request-certificate" signal, just set the client certificate upfront.
+        auto protectionSpace = WebCore::AuthenticationChallenge::protectionSpaceForClientCertificate(WebCore::soupURIToURL(soup_message_get_uri(msg)));
+        auto certificate = channel.session()->checkedNetworkStorageSession()->credentialStorage().get(m_request.cachePartition(), protectionSpace).certificate();
+        soup_message_set_tls_client_certificate(msg, certificate);
+    }
+
+    g_signal_connect(msg, "request-certificate-password", G_CALLBACK(+[](SoupMessage* msg, GTlsPassword* tlsPassword, WebSocketTask* task) -> gboolean {
+        auto protectionSpace = WebCore::AuthenticationChallenge::protectionSpaceForClientCertificatePassword(WebCore::soupURIToURL(soup_message_get_uri(msg)), tlsPassword);
+        auto password = task->protectedChannel()->session()->checkedNetworkStorageSession()->credentialStorage().get(task->m_request.cachePartition(), protectionSpace).password().utf8();
+        g_tls_password_set_value(tlsPassword, reinterpret_cast<const unsigned char*>(password.data()), password.length());
+        soup_message_tls_client_certificate_password_request_complete(msg);
+        return TRUE;
+    }), this);
 #endif
 
     soup_session_websocket_connect_async(session, msg, nullptr, protocols.get(), RunLoopSourcePriority::AsyncIONetwork, m_cancellable.get(),
@@ -96,7 +119,7 @@ WebSocketTask::WebSocketTask(NetworkSocketChannel& channel, const WebCore::Resou
 
     g_signal_connect(msg, "starting", G_CALLBACK(+[](SoupMessage* msg, WebSocketTask* task) {
         task->m_request.updateFromSoupMessageHeaders(soup_message_get_request_headers(msg));
-        task->m_channel.didSendHandshakeRequest(WTFMove(task->m_request));
+        task->protectedChannel()->didSendHandshakeRequest(WTFMove(task->m_request));
     }), this);
 }
 
@@ -108,6 +131,11 @@ WebSocketTask::~WebSocketTask()
     cancel();
 }
 
+Ref<NetworkSocketChannel> WebSocketTask::protectedChannel() const
+{
+    return m_channel.get();
+}
+
 String WebSocketTask::acceptedExtensions() const
 {
 #if SOUP_CHECK_VERSION(2, 67, 90)
@@ -117,7 +145,7 @@ String WebSocketTask::acceptedExtensions() const
         auto* extension = SOUP_WEBSOCKET_EXTENSION(it->data);
 
         if (!result.isEmpty())
-            result.append(", ");
+            result.append(", "_s);
         result.append(String::fromUTF8(SOUP_WEBSOCKET_EXTENSION_GET_CLASS(extension)->name));
 
         GUniquePtr<char> params(soup_websocket_extension_get_response_params(extension));
@@ -144,9 +172,10 @@ void WebSocketTask::didConnect(GRefPtr<SoupWebsocketConnection>&& connection)
     g_signal_connect_swapped(m_connection.get(), "error", reinterpret_cast<GCallback>(didReceiveErrorCallback), this);
     g_signal_connect_swapped(m_connection.get(), "closed", reinterpret_cast<GCallback>(didCloseCallback), this);
 
-    m_channel.didConnect(String::fromLatin1(soup_websocket_connection_get_protocol(m_connection.get())), acceptedExtensions());
+    Ref channel = m_channel.get();
+    channel->didConnect(String::fromLatin1(soup_websocket_connection_get_protocol(m_connection.get())), acceptedExtensions());
 
-    m_channel.didReceiveHandshakeResponse(m_handshakeMessage.get());
+    channel->didReceiveHandshakeResponse(m_handshakeMessage.get());
     g_signal_handlers_disconnect_by_data(m_handshakeMessage.get(), this);
     m_handshakeMessage = nullptr;
 }
@@ -156,15 +185,13 @@ void WebSocketTask::didReceiveMessageCallback(WebSocketTask* task, SoupWebsocket
     if (g_cancellable_is_cancelled(task->m_cancellable.get()))
         return;
 
-    gsize dataSize;
-    const auto* data = g_bytes_get_data(message, &dataSize);
-
+    std::span data = span(message);
     switch (dataType) {
     case SOUP_WEBSOCKET_DATA_TEXT:
-        task->m_channel.didReceiveText(String::fromUTF8(static_cast<const char*>(data), dataSize));
+        task->protectedChannel()->didReceiveText(String::fromUTF8(data));
         break;
     case SOUP_WEBSOCKET_DATA_BINARY:
-        task->m_channel.didReceiveBinaryData(static_cast<const uint8_t*>(data), dataSize);
+        task->protectedChannel()->didReceiveBinaryData(data);
         break;
     }
 }
@@ -182,20 +209,21 @@ void WebSocketTask::didFail(String&& errorMessage)
     if (m_receivedDidFail)
         return;
 
+    Ref channel = m_channel.get();
     m_receivedDidFail = true;
     if (m_handshakeMessage) {
-        m_channel.didReceiveHandshakeResponse(m_handshakeMessage.get());
+        channel->didReceiveHandshakeResponse(m_handshakeMessage.get());
         g_signal_handlers_disconnect_by_data(m_handshakeMessage.get(), this);
         m_handshakeMessage = nullptr;
     }
-    m_channel.didReceiveMessageError(WTFMove(errorMessage));
+    channel->didReceiveMessageError(WTFMove(errorMessage));
     if (!m_connection) {
         didClose(SOUP_WEBSOCKET_CLOSE_ABNORMAL, { });
         return;
     }
 
     if (soup_websocket_connection_get_state(m_connection.get()) == SOUP_WEBSOCKET_STATE_OPEN)
-        didClose(WebCore::WebSocketChannel::CloseEventCodeAbnormalClosure, { });
+        didClose(WebCore::ThreadableWebSocketChannel::CloseEventCodeAbnormalClosure, { });
 }
 
 void WebSocketTask::didCloseCallback(WebSocketTask* task)
@@ -214,10 +242,10 @@ void WebSocketTask::didClose(unsigned short code, const String& reason)
         return;
 
     m_receivedDidClose = true;
-    m_channel.didClose(code, reason);
+    protectedChannel()->didClose(code, reason);
 }
 
-void WebSocketTask::sendString(const IPC::DataReference& utf8, CompletionHandler<void()>&& callback)
+void WebSocketTask::sendString(std::span<const uint8_t> utf8, CompletionHandler<void()>&& callback)
 {
     if (m_connection && soup_websocket_connection_get_state(m_connection.get()) == SOUP_WEBSOCKET_STATE_OPEN) {
 #if SOUP_CHECK_VERSION(2, 67, 3)
@@ -225,13 +253,13 @@ void WebSocketTask::sendString(const IPC::DataReference& utf8, CompletionHandler
         GRefPtr<GBytes> bytes = adoptGRef(g_bytes_new_static(utf8.data(), utf8.size()));
         soup_websocket_connection_send_message(m_connection.get(), SOUP_WEBSOCKET_DATA_TEXT, bytes.get());
 #else
-        soup_websocket_connection_send_text(m_connection.get(), CString(reinterpret_cast<const char*>(utf8.data()), utf8.size()).data());
+        soup_websocket_connection_send_text(m_connection.get(), CString(utf8).data());
 #endif
     }
     callback();
 }
 
-void WebSocketTask::sendData(const IPC::DataReference& data, CompletionHandler<void()>&& callback)
+void WebSocketTask::sendData(std::span<const uint8_t> data, CompletionHandler<void()>&& callback)
 {
     if (m_connection && soup_websocket_connection_get_state(m_connection.get()) == SOUP_WEBSOCKET_STATE_OPEN)
         soup_websocket_connection_send_binary(m_connection.get(), data.data(), data.size());
@@ -250,7 +278,7 @@ void WebSocketTask::close(int32_t code, const String& reason)
     }
 
 #if SOUP_CHECK_VERSION(2, 67, 90)
-    if (code == WebCore::WebSocketChannel::CloseEventCodeNotSpecified)
+    if (code == WebCore::ThreadableWebSocketChannel::CloseEventCodeNotSpecified)
         code = SOUP_WEBSOCKET_CLOSE_NO_STATUS;
 #endif
 
