@@ -32,11 +32,11 @@
 #include "IPCStreamTesterIdentifier.h"
 #include "MessageReceiver.h"
 #include "ScopedActiveMessageReceiveQueue.h"
-#include "SharedMemory.h"
 #include "StreamConnectionBuffer.h"
 #include "StreamConnectionWorkQueue.h"
 #include "StreamMessageReceiver.h"
 #include "StreamServerConnection.h"
+#include <WebCore/SharedMemory.h>
 #include <atomic>
 #include <wtf/HashMap.h>
 #include <wtf/WorkQueue.h>
@@ -45,37 +45,41 @@
 
 namespace WebKit {
 
-#define ASSERT_IS_TESTING_IPC() ASSERT(WebKit::isTestingIPC(), "Untrusted connection sent invalid data. Should only happen when testing IPC.")
-
 #if ENABLE(IPC_TESTING_API)
 
-// Function to check when asserting IPC-related failures, so that IPC testing skips the assertions
-// and exposes bugs underneath.
-bool isTestingIPC();
-
 class IPCConnectionTester;
 class IPCStreamTester;
 
 // Main test interface for initiating various IPC test activities.
-class IPCTester final : public IPC::MessageReceiver {
+class IPCTester final : public IPC::MessageReceiver, public RefCounted<IPCTester> {
 public:
-    IPCTester();
+    static Ref<IPCTester> create();
     ~IPCTester();
 
+    void ref() const final { RefCounted::ref(); }
+    void deref() const final { RefCounted::deref(); }
+
     // IPC::MessageReceiver overrides.
     void didReceiveMessage(IPC::Connection&, IPC::Decoder&) final;
     bool didReceiveSyncMessage(IPC::Connection&, IPC::Decoder&, UniqueRef<IPC::Encoder>&) final;
 private:
+    IPCTester();
+
     // Messages.
     void startMessageTesting(IPC::Connection&, String&& driverName);
-    void stopMessageTesting(CompletionHandler<void()>);
-    void createStreamTester(IPC::Connection&, IPCStreamTesterIdentifier, IPC::StreamConnectionBuffer&&);
+    void stopMessageTesting(CompletionHandler<void()>&&);
+    void createStreamTester(IPC::Connection&, IPCStreamTesterIdentifier, IPC::StreamServerConnection::Handle&&);
     void releaseStreamTester(IPCStreamTesterIdentifier, CompletionHandler<void()>&&);
-    void createConnectionTester(IPC::Connection&, IPCConnectionTesterIdentifier, IPC::Attachment&&);
-    void createConnectionTesterAndSendAsyncMessages(IPC::Connection&, IPCConnectionTesterIdentifier, IPC::Attachment&&, uint32_t messageCount);
+    void createConnectionTester(IPC::Connection&, IPCConnectionTesterIdentifier, IPC::Connection::Handle&&);
+    void createConnectionTesterAndSendAsyncMessages(IPC::Connection&, IPCConnectionTesterIdentifier, IPC::Connection::Handle&&, uint32_t messageCount);
     void releaseConnectionTester(IPCConnectionTesterIdentifier, CompletionHandler<void()>&&);
     void sendSameSemaphoreBack(IPC::Connection&, IPC::Semaphore&&);
     void sendSemaphoreBackAndSignalProtocol(IPC::Connection&, IPC::Semaphore&&);
+    void sendAsyncMessageToReceiver(IPC::Connection&, uint32_t);
+    void asyncPing(uint32_t value, CompletionHandler<void(uint32_t)>&&);
+    void syncPing(IPC::Connection&, uint32_t value, CompletionHandler<void(uint32_t)>&&);
+    void syncPingEmptyReply(IPC::Connection&, uint32_t value, CompletionHandler<void()>&&);
+    void asyncOptionalExceptionData(IPC::Connection&, bool sendEngaged, CompletionHandler<void(std::optional<WebCore::ExceptionData>, String)>&&);
 
     void stopIfNeeded();
 
@@ -89,13 +93,6 @@ private:
     ConnectionTesterMap m_connectionTesters;
 };
 
-#else
-
-constexpr inline bool isTestingIPC()
-{
-    return false;
-}
-
 #endif
 
 }
